(*The Package*)

(*Title : SysModel, a Mathematica package for solving system of differential 
equations  with application for modeling Biokinetic Systems*)


(* : Author : Guillermo Sanchez*)


(* : Summary : This package provides Mathematica functions for solving linear compartmental and physiological model*)


(* : Context : Biokmod`SysModel`*)


(* : Package Version : 1.3 *)

(* : Copyright : Copyright 2005. Guillermo Sanchez. http : //web.usal.es/~guillerm/*)
 
(* : History : Version 1.1 added SystemLTSolve; v.1.2 added sseH2,sseH3, and menors 
update;v.1.3 moved ssH feature for fitting to a new package*)

(* : Keywords : Differential equation system, Compartmental Model, Physiological Model, Biokinetic, 
Pharmacokinetic, Internal Dosimetry, Radiological Protection*)
 
(*Sources : Jacquez 1985, Andersen 1983, Godfrey 1983*)

(* : Warning :*)

(* : Mathematica Version : 5.1*)

(* : Limitation : None known.*)

(* : Discussion :*)

BeginPackage["Biokmod`SysModel`"]

CoefMatrix::usage = "CoefMatrix[n,{coeffs}], gives the matrix of 
coefficients, for n number of retention variables and coeffs are coefficients 
of the matrix. They are given as {{..{i,j,aij},{..},...} where ij is 
position of the coef. in the matrix (i=file, j=column) and are the values 
of this coeff (By default aij = 0).";

CompartMatrix::usage = "CompartMatrix[n,{transcoeff},lambda], gives the
matrix of coefficients,also called  constant transfer coefficients matrix. 
Where n isthe number of compartments and transcoeff are the transfer coefficients, also 
called clearance coeffs. or dissolution rates. They are given as 
{{..{i,j,kij},{..},...} where kij is the transfer coeff., in \!\(t\^-1\),. 
from compartment i to compartment j (By default kij = 0); lambda is the 
radioactive decay constant, in the same unit as the trans. coef. (by default lambda = 0, which 
means thatit is not a radioactive substance.)";

matrixExp::usage = "matrixExp[matrixA,t] gives the exponential matrix Exp[A t]";

AcuteInput::usage = "AcuteInput[MatrixCoefs, input, t, x] gives the 
retention in each compartment i at time t (in the same unit that the 
1/transfer of coeffs.) after of a single input {b1,..,bi,..,bn} in the 
compartments {1,..,i,..n} at time t = 0. MatrixCoefs is the coeffs matrix 
{{..,aij,..},{..},...}. It can be given directly or better obtained by using 
the function of this package CompartMatrix or CoefMatrix; x is the symbol 
that represents the retention variables.";

MultiExpInput::usage = "MultiExpInput[MatrixCoefs,incond,input,t, t1, x] 
gives the retention in each compartment i at time t, in the same unit that 
the 1/transfer of coeffs., assuming a multiexponential input {a11 Exp[b11 t]+
...+a1i Exp[b1it],...,an1 Exp[bn1 t]+..+ani Exp[bnit]}; incond \
{c1,..,ci,..cn} are the initial conditions in compartments {1,..,i,..,n}. MatrixCoefs is the 
coeffs matrix {{..,aij,..},{..},...}. It can be given directely or better obtained 
using the function of this package CompartMatrix or CoefMatrix; x is the symbol 
that represents the retention variables.";

SystemDSolve::usage = "SystemDSolve[MatrixCoefs, incond, input, t, t1, x, options] gives 
the solution X(t)= {x1(t), ..., xn(t) at time t1 of a ODE system X'(t) = A X(t) + B(t), 
with initial conditions X(0) = IC , where MatrixCoefs is the 
coeffs matrix A = {{a11,...a1n}, ...,{am1, ..., amn}}; incond are IC = {c1,..,ci,..cn}; 
B(t) ={b1[t],..., bn[t]};  x is the symbol that represents X; with options 
can be choose the method applied for solving the SODE, by default is apply: Method-> 
Automatic, other methods are LaplaceTransform, EigenValues, is applied the 
Eigenvalues-eigensisystem method, and DSolve is applied the Mathematica function DSolve.";

SystemNDSolve::usage = "SystemNDSolve[MatrixCoefs, incond, input, {t, tmin, tmax}, t1, x, options] 
gives the numeric solution X(t)= {x1(t), ..., xn(t) at time t1 of a ODE 
system X'(t) = A X(t) + B(t), with initial conditions X(0) = IC , where; MatrixCoefs is the 
coeffs matrix A = {{a11,...a1n}, ...,{am1, ..., amn}}; incond are IC = {c1,..,ci,..cn}; 
B(t) ={b1[t],..., bn[t]};  x is the symbol that represents X; options are the 
options of NDSolve. 
Note:  This function apply the Mathematica function NDSolve  for solving the 
SODE, it should be used when MatrixCoefs included some coeffs that are not constants 
or when it was not possible obtain the solution of the model using SystemDSolve.";

SystemLTSolve::usage = "SystemLTSolve[MatrixCoefs, incond, input, t, s, X, options] gives  
the laplace transform solution X(s)= {X1(s), ..., Xn(s)}  of a ODE system x'(t) = A x(t) + b(t),  with initial conditions x(0)  where MatrixCoefs is A = {{a11,...a1n}, ...,{am1, ..., amn}}, incond are initial conditions x(0) = {x1(0),....,xn(0)} , and input is b(t) ={b1[t],..., bn[t]}";

Catenary::usage = "Catenary[b, n, t, K, k, lambda] gives the amount 
(activity,mass,concentration, etc.) in the n-th compartment of a one-way 
catenary chain at any time t after a single intake b in the first compartment
of the chain, K is the general symbol applied for indicate the total transfer since one 
compartment and k notate the transfer since any compartments of the catenary chain; 
lambda is the radioactive decay constant, in the same unit that the trans. coef. 
(by default i lambda = 0, which means it that is not a radioactive substance. 
Note: The transfer coefficients are given as ki and Ki where ki is the transfer coeffs, 
in \!\(t\^-1\), from compartment i to compartment i+1 and Ki is the total transfer coeffs. 
from i to others. ";

ShowODE::usage = "ShowODE[MatrixCoefs,incond,input,t, x] gives system of 
ODE for a compartmental or physiological  model where MatrixCoefs is the 
coeffs matrix {{..,aij,..},{..},...}, it can be given directly  or better obtained by usingthe 
function of this package CompartMatrix or CoefMatrix; incond {c1,..,ci,..cn} are 
the initial conditions in compartments {1,..,i,..,n}; input are the inputs 
{b1[t],..., bn[t]} in compartment {1, .., n}; x is the symbol that 
represents the retention variables.";

Options[SystemDSolve]=Method\[Rule]"Automatic";

Options[SystemNDSolve]= Options[NDSolve];

Begin["`Private`"]

CompartMatrix[(n_)?IntegerQ, lst_?MatrixQ, lambda_:0] := 
   Module[{k, A}, Apply[Set, ({k @@ Take[#1, 2], Last[#1]} & ) /@ lst, {1}]; 
     k[i_, j_] := 0; A = DiagonalMatrix[Table[-Sum[k[i, j], {j, 0, n}], 
         {i, 1, n}]] + Table[k[j, i], {i, 1, n}, {j, 1, n}] - 
       lambda*IdentityMatrix[n]]; 

CoefMatrix[(n_)?IntegerQ, (lst_)?MatrixQ] := 
  Module[{i, j, k}, Normal[SparseArray[lst /. {i_, j_, k_} -> {i, j} -> k, {n, n}]]];

matrixExp[matrixA_?MatrixQ, t_] := Module[{eval, evec, m1}, 
   {eval, evec} = Eigensystem[matrixA]; 
    m1 = Transpose[evec] . DiagonalMatrix[Exp[eval*t]] . 
      Inverse[Transpose[evec]]];

AcuteInput[matrixA_?MatrixQ, input_List, t_, x_] := 
   Module[{r}, r = Dot[matrixExp[matrixA, t],input]; 
     Thread[Subscript[x, #1][t]& /@ Range[Length[input]] -> r]]


MultiExpInput[matrixA_List, incond_List, input_List, t_, t1_, x_] := 
   Module[{m1, m2, r, tau, a, b, c}, m1 = matrixExp[matrixA, t]; 
     m2 = Chop[ExpandAll[Evaluate[m1 /. t -> tau] . input /. 
            t -> t - tau]] /. E^((b_.)*t + (c_.)*tau + (d_.)*tau) -> 
          E^(b*t + (c + d)*tau) /. Exp[(c_.)*tau] -> (E^(c*t) - 1)/c /. 
       E^((b_.)*t + (c_.)*tau) -> (E^(b*t)*(E^(c*t) - 1))/c; 
     r = ExpandAll[m1 . incond + m2]; 
     Thread[Subscript[x, #1][t]& /@ Range[Length[input]] -> r] /. 
      t -> t1]/;Plus@@(input/.a_.*Exp[b_.*t]/;FreeQ[{a,b},t]->0)===0;


SystemDSolve["Automatic",matrixA_,incond_List,input_List,t_,t1_,x_,
opts___?OptionQ]/;MatrixQ[matrixA, NumericQ]:=
    Module[{integrate,t2,y,z,a,b,c,d,tau,m1,m2,m3,m4,r},
      integrate[y_+z_,t2_]:=integrate[y,t2]+integrate[z,t2];
      integrate[c_ y_,t2_]:=c*integrate[y,t2]/;FreeQ[c,t2]&&FreeQ[c,tau];
      integrate[E^((b_.)*t2_+(c_.)*tau),t2_]:=(E^(b*t2)*(E^(c*t2)-1))/c/;
          FreeQ[{b,c},t2]&&FreeQ[{b,c},tau];
      integrate[Exp[(a_.)*tau], t2_]:=(E^(a*t2)-1)/a/;
          FreeQ[a,t2]&&FreeQ[a,tau];
      integrate[c_,t2_]:=c*t2/;FreeQ[c,t2]&&FreeQ[c,tau];
      integrate[a_,t2_]:=Integrate[a,{tau,0,t2}];
      m1=matrixExp[matrixA,t];m2=m1/.t\[Rule]tau;
      m3=input/.t\[Rule]t-tau;
      m4=Chop[ExpandAll[m2.m3]]/.E^((b_.)*t2_+(c_.)*tau+(d_.)*tau)\[Rule]
            E^(b*t2+(c+d)*tau);
      r=Chop[ExpandAll[m1.incond+Map[integrate[#1,t]&,m4,{1}]]];
     Thread[Subscript[x, #1][t]& /@ Range[Length[input]] -> r]/.t\[Rule]t1];



SystemDSolve["LaplaceTransform",matrixA_?MatrixQ, incond_List, input_List, t_, t1_, x_,
opts___?OptionQ] := Module[{B, A1, R, P, r, n, s}, A1 = matrixA; n = Dimensions[A1][[1]]; 
    B = s*IdentityMatrix[n] - A1; R = Inverse[B]; P = Apart[R]; 
    r = Table[LaplaceTransform[Extract[input, i], t, s], {i, 1, n}]; 
    R = Chop[ExpandAll[InverseLaplaceTransform[ P,s,t].incond + 
    InverseLaplaceTransform[P . r, s, t]]]; 
    Thread[Subscript[x, #1][t]& /@ Range[n] -> R]/. t -> t1];

SystemDSolve["EigenValues",matrixA_?MatrixQ,incond_List,input_List,t_,t1_,x_,opts___?OptionQ]:=
    Module[{integrate,t2,y,z,a,b,c,d,tau,m1,m2,m3,m4,r},
      integrate[y_+z_,t2_]:=integrate[y,t2]+integrate[z,t2];
      integrate[c_ y_,t2_]:=c*integrate[y,t2]/;FreeQ[c,t2]&&FreeQ[c,tau];
      integrate[a_,t2_]:=Integrate[a,{tau,0,t2}];
      m1=matrixExp[matrixA,t];m2=m1/.t\[Rule]tau;
      m3=input/.t\[Rule]t-tau;
      m4=Chop[ExpandAll[m2.m3]]/.E^((b_.)*t2_+(c_.)*tau+(d_.)*tau)\[Rule]
            E^(b*t2+(c+d)*tau);
      r=Chop[ExpandAll[m1.incond+Map[integrate[#1,t]&,m4,{1}]]];
      Thread[Subscript[x, #1][t]& /@ Range[Length[input]] -> r]/.t\[Rule]t1];


SystemDSolve["DSolve",matrixA_?MatrixQ, incond_List, input_List, t_, t1_, x_,opts___?OptionQ] := 
  Module[{B, A1, R, X, n, i}, A1 = Rationalize[matrixA]; 
     n = Dimensions[A1][[1]]; 
     R = Join[Thread[Table[Derivative[1][Subscript[x, i]][t], {i, 1, n}] == 
         Table[Subscript[x, i][t], {i, 1, n}] . Transpose[A1] + 
          Rationalize[input]], 
          Thread[Table[Subscript[x, i][0], {i, 1, n}] == 
         Rationalize[incond]]]; B = Table[Subscript[x, i][t], {i, 1, n}]; 
     X = Flatten[DSolve[R, B, t]]; Chop[ExpandAll[X]] /. t -> t1];

SystemDSolve[matrixA_?MatrixQ, incond_List, input_List, t_, t1_,x_,opts___?OptionQ] :=  
Module[{meth},meth=Method/.{opts}/.Options[SystemDSolve];
SystemDSolve[meth,matrixA, incond, input, t, t1, x, opts]];

Catenary[b_, n_, t_, K_, k_, lambda_:0] := 
  Block[{p, j}, 
   (b*Sum[1/(E^(Subscript[K, j]*t)*(Product[Subscript[K, p] - 
           Subscript[K, j], {p, 1, j - 1}]*Product[Subscript[K, p] - 
           Subscript[K, j], {p, j + 1, n}])), {j, 1, n}]*
     Product[Subscript[k, j], {j, 1, n - 1}])/E^(lambda*t)];

ShowODE[matrixA_, incond_, input_, t_, x_] := Module[{n, n1, A, B, c}, 
   n = Dimensions[matrixA][[1]]; n1 = Range[n]; 
    A = (Derivative[1][Subscript[x, #1]][t] & ) /@ n1; 
    B = (Subscript[x, #1][t] & ) /@ n1 . Transpose[matrixA] + input; 
    c = (Subscript[x, #1][0] & ) /@ n1; Join[Thread[A == B], 
     Thread[c == incond]]];

SystemNDSolve[matrixA_, incond_, input_, {t_, tmin_, tmax_}, t1_, x_, opts___] := 
Module[{B, A1, R, P, n, i, opt1}, 
   A1 = matrixA; n = Dimensions[A1][[1]]; 
    R = Join[Thread[Table[Derivative[1][Subscript[x, i]][t], {i, 1, n}] == 
        Table[Subscript[x, i][t], {i, 1, n}] . Transpose[matrixA] + input], 
      Thread[Table[Subscript[x, i][0], {i, 1, n}] == incond]]; 
    B = Table[Subscript[x, i][t], {i, 1, n}]; 
    Flatten[NDSolve[R, B, {t, tmin, tmax}, opts]] /. t -> t1];

SystemLTSolve[matrixA_List,incond_List, input_List, t_, s_, X_] := 
  Module[{B, A1, R, R1, P, r, n}, A1 = matrixA; n = Dimensions[A1][[1]]; 
    B = s*IdentityMatrix[n] - A1; R = Inverse[B]; P = Apart[R]; 
    r = LaplaceTransform[input, t, s]; 
    R1 = Dot[P, incond] + Dot[P, r]; 
    Thread[Subscript[X, #1][s]& /@ Range[n] -> R1]];


End[]

Protect[CoefMatrix, CompartMatrix, matrixExp, AcuteInput, MultiExpInput, SystemDSolve, 
    Catenary, ShowODE,  SystemNDSolve, SystemLTSolve]; 

SetAttributes[Subscript, NHoldRest]; 

Print["SysModel, guillermo@usal.es, version 1.3.   2005-08-24"]

EndPackage[]
