(*Title : HUMORAP, A package for solving ICRP 66, 78 and other compartmental models*)

(* : Author : Guillermo Sanchez *)

(* : Summary :
This package provides analytical and numerical method to solve compartmental 
models with spetial application in radioprotection. It included the ICRP 66 and 78 models.
It has been developped with the support of ENUSA Industrias Avanzadas,  
http : // www.enusa.es*)

(* : Context : `Humorap3` *)

(* : Package Version : 3.3*)

(* : Copyright : Copyright 2004, 
  Guillermo Sanchez. guillermo@usal.es or gsl@fab.enusa.es *)

(* : History : Version 1.0, January 2001.  Version 1.1, July 2001,  
  Version 1.1.1, September 2001,  Version 2.0  June 2002, Version 2.1  July 2002, 
  Version 2.2  October 2002, Version 2.3,  January 2003, Version 2.3.1  April 2003,
  Version 2.4 July 2003, Version 3.0 January 2004,  Version 3.1 March 2004, 
  Version 3.2 September 2004, Version 3.3 June 2005*)

(* : Keywords : Compartmental Models, ICRP 66, ICRP78*)

(* : Sources : ICRP 66 and ICRP 78*)

(* : Warning :  *)

(* : Mathematica Version : 5.1*)

(* : Limitation : None known. *)

(* : Discussion : *)

BeginPackage["Biokmod`Humorap3`", "Biokmod`Resptract`","Biokmod`SysModel`", "Biokmod`Master`",
"Graphics`Graphics`","Graphics`Legend`"]

qRT::usage = "qRT[int, {AI, bbfastseq, bbslow, BBfastseq, BBslow},
{\!\(s\_p, s\_pt, s\_t\)}, t1, lambda, options], for one specific nuclide, 
gives the  retention in each compartment of ICRP66 model at time t, in days, 
for a single intake <int> in t = 0. AI, bbfastseq, bbslow, BBfastseq, BBslow are 
the initial fractional deposition in AI,  \!\(bb\_(fast+seq)\),  \!\(bb\_2(slow)\),  
\!\(BB\_(fast+seq)\)  and   \!\(BB\_2(slow)\).  (Recommended fractional 
deposition values for reference workers as a function of aerosol size can 
be obtained from Annex F of ICRP 66). Lungs include AI, bb, BB, \!\(LN\_TH\). 
\!\(s\_p, s\_pt, s\_t\) are the dissolution rates and lambda is the decay 
constant, in \!\(days\^-1\) . Information about the options, 
including the default values, can be shown with Options[qRT]. 
Note: \!\(bb\_(fast+seq)\) = \!\(bb\_1(fast)\) + \!\(bb\_seq\), bb = 
\!\(bb\_(fast+seq)\) + \!\(bb\_2(slow)\), \!\(BB\_(fast+seq)\) = 
\!\(BB\_1(fast)\) + \!\(BB\_seq\), and BB = \!\(BB\_(fast+seq)\) + 
\!\(BB\_2(slow)\).";

qRTandGI::usage = "qRTandGI[int, {AI, bbfastseq, bbslow, BBfastseq, BBslow, ET2, ET1}, 
{sp, spt, ss}, f1, t1,  opts], for one specific nuclide, 
gives the retention in Lungs, B (transferred to blood directly from Respiratory Tract and 
from GI), ST , SI, ULI, LLI, and FEC as function of time at time t1 after an acute 
intake in t=0, it is assumed lambda -> 0";

qGI::usage = "qGI[int, f1, t1, lambda, options], for one specific nuclide gives 
the retention in compartments of the GI Tract as function of time at time t1 
after an acute intake by ingestion in t = 0. (Transferences from B to GI are 
not be account)";

qContinuous::usage = "qContinuous[continuous,input-inpulse,t,t1] gives the 
amount (activity, mass, etc) in one compartment or region (i.e Lungs) of 
ICRP66-Extended system for a continuous intake b(t) during a time t1 where is 
known the retention function for a single intake 1 in t=0 (it can be 
calculated using AcuteInput).";

qInhalation::usage = "qInhalation[Compartmental Matrix,Int,IFD, FRA, f1, t1, lambda,
options] for one specific nuclide, gives the retention in compartments defined in 
Compartmental Matrix (ST and SI not included)as function of time at time t1 after 
the intake. An acute intake is assumed by default. With Options  can be chosen other 
intake types: Constant and Continuous";

qInhalationE::usage = "qInhalationE[CCompartmentalMatrixExpanded,Int,IFD,FRA,f1, t1, 
lambda,options] for one specific nuclide, gives the retention in compartments defined 
in Compartmental Matrix Extended (ST and SI much be included) as function of time at 
time t1 after intake. An acute intake is assumed by default. With Options  can be chosen other 
intake types: Constant and Continuous ";

qIngestion::usage = "qIngestion[Compartmental Matrix, Int, f1, t1, lambda,
options]for one specific nuclide, gives the retention in compartments defined in 
Compartmental Matrix as function of time. An acute intake is assumed by default. 
With Options  can be chosen other intake types: Constant, Continuous and MultiInputs Intake,
in this last case inp much be written {{I0, t0},...,{In, tn}}, then t1 is the time after the 
last intake, so for t1 is taken tn = 0 ";

qIngestionE::usage = "qIngestionE[Compartmental Matrix, int,f1,t1, lambda,
options]for one specific nuclide, gives the retention in compartments defined in 
Compartmental Matrix Extended  (ST and SI much be included) as function of time.
An acute intake is assumed by default. With Options  can be chosen other intake types: 
Constant, Continuous and MultiInputs Intake,in this last case inp much be written 
{{I0, t0},...,{In, tn}}, then t1 is the time after the last intake, so for t1 is taken tn = 0 ";

qWholebody::usage = "qWholebody[CompartmentalMatrix, I0, f1, t1, lambda, options]
gives the whole body retention  at time t1 after an acute intake I0 by ingestion t = 0; 
qWholebody[CompartmentalMatrix,I0, IFD, FRA, f1, t1, lambda, options] gives the whole body 
retention at time t1 after an acute intake I0 by inhalation t = 0";

qWholebodyE::usage = "qWholebodyE[CompartmentalMatrixExpanded, I0, f1, t1, lambda, options]
gives the whole body retention  at time t1 after an acute intake I0 by ingestion t = 0; 
qWholebodyE[CompartmentalMatrixExpanded,I0, IFD, FRA, f1, t1, lambda, options] gives 
the whole body retention at time t1 after an acute intake I0 by inhalation t = 0.
Note:  ST and SI much be included in Compartmental Matrix Extended";

qInjection::usage = "qInjection[Compartmental Matrix, int, t1, lambda, options] for 
one specific nuclide, gives the retention in compartments defined in Compartmental Matrix 
as function of time after an intake int by injection in the blood. An acute intake is assumed by default. 
With Options can be chosen other intake types: Constant, Continuous and MultiInputs Intake,
in this last case inp much be written {{I0, t0},...,{In, tn}}, then t1 is the time after the 
last intake, so for t1 is taken tn = 0 "; 

qInjectionE::usage = "qInjectionE[CompartmentalMatrixExpanded, int, f1, t1, lambda, opts] for 
one specific nuclide, gives the retention in compartments defined in Compartmental Matrix 
Extended (ST and SI much be included) as function of time after an intake int by injection in the blood. 
An acute intake is assumed by default. With Options can be chosen other intake types: Constant, 
Continuous and MultiInputs, in this last case inp much be written {{I0, t0},...,{In, tn}}, 
then t1 is the time after the last intake, so for t1 is taken tn = 0 "; 

BioassayReport::usage = "BioassayReport[CompartmentalMatrix, I0, f1, t1, lambda, options]
gives the daily urine and faecal excretion and the whole body retention for one specific nuclide, at time t1 
after an intake I0 by ingestion; BioassayReport[CompartmentalMatrix,I0, IFD, FRA, f1, t1, lambda, options] gives 
the lung retention and the daily urine and faecal excretion and the whole body retention for 
one intake by inhalation. An acute intake is assumed by default. With Options can be chosen other intake types: 
Constant, Continuous and MultiInputs, in this last case inp much be written {{I0, t0},...,{In, tn}}, 
then t1 is the time after the last intake, so for t1 is taken tn = 0 "; 

BioassayReportE::usage = "BioassayReportE[CompartmentalMatrixExpanded, I0, f1, t1, lambda, 
options] gives the daily urine and faecal excretion and the whole body retention for one specific nuclide, 
at time t1 after an intake I0 by ingestion; BioassayReportE[CompartmentalMatrixExpanded,I0, IFD, FRA, f1, t1, lambda, 
options] gives the lung retention and the daily urine and faecal excretion and the whole body retention for 
one intake by inhalation. An acute intake is assumed by default. With Options can be chosen other intake types: 
Constant, Continuous and MultiInputs, in this last case inp much be written {{I0, t0},...,{In, tn}}, 
then t1 is the time after the last intake, so for t1 is taken tn = 0.
Note:  ST and SI much be included in Compartmental Matrix Extended"; 

icrp30Model::usage = "ICRP30Model[fu,ff,{Ti},Ci,Tb}] gives the transfer rate 
for a ICRP 30 model where fu,ff,Ti,Ci,Tb can be obtained in Health Phys 
83:775-779(tableE3) and CD-ROM ICRP78"; 

qRandom::usage = "qRandom[b1, T, inp, mu, sig, z, n, m , t_:t] 
gives the IRF for an random imput with mean mu and standard deviation sig."; 

Options[qRT] = Options[qRespTract];

Options[qGI] = {Subscript[k, ST] -> 24, Subscript[f, B] -> f1/(1 - f1), 
   Subscript[k, SI] -> 6, Subscript[k, ULI] -> 1.8, Subscript[k, LLI] -> 1};

Options[qRTandGI] = Flatten[{Options[qRespTract],Options[qGI]}];

Options[qInjection] = {IntakeType -> "Acute", Subscript[k, ULI] -> 1.8, 
    Subscript[k, LLI] -> 1}; 

Options[qInjectionE] = {IntakeType -> "Acute", Subscript[k, ST] -> 24,
Subscript[f, B] -> f1/(1 - f1), Subscript[k, SI] -> 6, Subscript[k, ULI] -> 1.8, 
Subscript[k, LLI] -> 1};

Options[qIngestion] =  Flatten[{IntakeType -> "Acute", Options[qGI]}];

Options[qIngestionE] =   Flatten[{IntakeType -> "Acute", Options[qGI]}];

Options[qInhalation] = Flatten[{IntakeType -> "Acute", Options[qRespTract],Options[qGI]}];

Options[qInhalationE] = Flatten[{IntakeType -> "Acute",Options[qRespTract],Options[qGI]}];

Options[qWholebody] = Flatten[{IntakeWay-> "Inhalation",Options[qRespTract],Options[qGI]}];

Options[qWholebodyE] = Flatten[{IntakeWay-> "Inhalation",Options[qRespTract],Options[qGI]}];

Options[BioassayReport] = Flatten[{IntakeWay-> "Inhalation",IntakeType-> "Acute", 
ShowReport-> "Automatic", Options[qRespTract],Options[qGI]}];

Options[BioassayReportE] = Flatten[{IntakeWay-> "Inhalation",IntakeType-> "Acute",
ShowReport->"Automatic", Options[qRespTract],Options[qGI]}];


Begin["`Private`"]

(* To be used in graphics options of bioassay reports*)

styles = {{AbsoluteThickness[0.5], AbsoluteDashing[{4, 4}], 
RGBColor[1, 0, 0]}, {RGBColor[0, 0, 1]}};
styles1 = {{AbsoluteThickness[0.5], AbsoluteDashing[{4, 4}], RGBColor[1, 0, 0]}, 
{AbsoluteThickness[1.5], RGBColor[0, 1, 0]}, {AbsoluteThickness[2], AbsoluteDashing[{1, 8}],
RGBColor[0, 0, 1]}};

(* Functions, see package help *)


icrp30Model[fu_, ff_, ti_List, fi_List, TB_] := 
   Module[{a, fR, h, n, i, k1}, a = Log[2]/ti; fR = 1 - Plus @@ fi; 
      n = Length[ti]; h = Flatten[{{fi, fR*fu, fR*ff}*(Log[2]/TB), 
         {fu*a, ff*a}, {12, Subscript[k, ULI], Subscript[k, LLI]}}]; 
      k1 = Flatten[{Table[{1, 1 + i}, {i, n + 1}], {{1, n + 4}}, 
         Table[{1 + i, 2 + n}, {i, n}], Table[{1 + i, 2 + n + 2}, {i, n}], 
         {{n + 2, n + 3}}, {{n + 4, n + 5}}, {{n + 5, n + 6}}}, 1]; 
      Transpose[Append[Transpose[k1], h]]]; 

qContinuous[input_, acute_List, t_, t1_] := Module[{integrate, t2, y, z, a, b, c, d, tau, m1, 
     m2, m3, m4, r, i}, integrate[y_ + z_, t2_] := integrate[y, t2] + integrate[z, t2]; 
     integrate[c_*y_, t2_] := c*integrate[y, t2] /; FreeQ[c, t2] && FreeQ[c, tau]; 
     integrate[E^((b_.)*t2_ + (c_.)*tau), t2_] := (E^(b*t2)*(E^(c*t2) - 1))/c /; 
       FreeQ[{b, c}, t2] && FreeQ[{b, c}, tau]; integrate[Exp[(a_.)*tau], t2_] := 
      (E^(a*t2) - 1)/a /; FreeQ[a, t2] && FreeQ[a, tau]; integrate[c_, t2_] := 
      c*t2 /; FreeQ[c, t2] && FreeQ[c, tau]; integrate[a_, t2_] := 
      Integrate[a, {tau, 0, t2}] /;  !FreeQ[a, tau]; m1 = acute; m2 = m1 /. t -> tau; 
     m3 = input /. t -> t - tau; m4 = Chop[ExpandAll[m2*m3]] /. 
       E^((b_.)*t2_ + (c_.)*tau + (d_.)*tau) -> E^(b*t2 + (c + d)*tau); 
     r = Chop[ExpandAll[(integrate[#1, t] & ) /@ m4]]; r /. t -> t1]; 

(*INJECTION*)

(* New in version 3.0*)

qInjection1[matrixA_, int_, t1_, lambda_, opts___Rule] := 
Module[{m, n, r, i}, n = Length[matrixA]; m = matrixA /. {opts} /. Options[qInjection]; 
r = Last /@ AcuteInput[m, Flatten[{1, PadLeft[{0}, n - 1]}], t1, q]; 
Thread[Flatten[{qB[t1], Table[Subscript[q, i][t1], {i, 2, n - 4}], qURI[t1], qULI[t1], 
qLLI[t1], qFEC[t1]}] -> Chop[ExpandAll[Flatten[int*r*Exp[N[-lambda]*t1]]], 10^(-45)]]]; 

qInjectionChronic[matrixA_, int_, t1_, lambda_, opts___Rule] := 
Module[{a, b, c, e, x1, x2, x3, t, t2}, a = qInjection1[matrixA, int, t2, N[lambda], opts]; 
{b, c} = {qURI[t2], qFEC[t2]} /. a; d = Plus @@ Last /@ a; e = d - b - c; 
{x1[t_], x2[t_], x3[t_]} = {First /@ a, Last /@ a, e} /. t2 -> t; 
Thread[Flatten[{x1[t1], qWB[t1]}] -> Flatten[(qContinuous[1, {#1}, t, t1] & ) /@ 
Flatten[{x2[t], x3[t]}]]]];

qInjectionContinuous[matrixA_, {int_, t3_}, t1_, lambda_, opts___Rule] := 
Module[{a, b, c, e, x1, x2, x3, t, t2}, a = qInjection1[matrixA, 1, t2, N[lambda], opts]; 
{b, c} = {qURI[t2], qFEC[t2]} /. a; d = Plus @@ Last /@ a; e = d - b - c; 
{x1[t_], x2[t_], x3[t_]} = {First /@ a, Last /@ a, e} /. t2 -> t; 
Thread[Flatten[{x1[t1], qWB[t1]}] -> Flatten[(qContinuous[int, {#1}, t3, t1] & ) /@ 
Flatten[{x2[t3], x3[t3]}]]]]; 

qInjection["Acute",matrixA_, int_, t1_, lambda_, opts___Rule] := 
  qInjection1[matrixA, int, t1, lambda, opts]  

qInjection["Constant",matrixA_, int_, t1_, lambda_, opts___Rule] := 
     qInjectionChronic[matrixA, int, t1, lambda, opts]; 

qInjection["Continuous",matrixA_, {int_, t3_}, t1_, lambda_, opts___Rule] := 
     qInjectionContinuous[matrixA, {int, t3}, t1, lambda, opts]; 

qInjection[matrixA_, int_, t1_, lambda_, opts___Rule] := 
   Module[{meth}, meth = IntakeType /. {opts} /. Options[qInjection]; 
     qInjection[meth, matrixA, int, t1, lambda, opts]]; 

qInjection[(matrixA_)?MatrixQ, int_, t1_, lambda_, {(opts___)?OptionQ}]:=
"Something is wrong"; 

qInjectionE1[matrixA_, int_, fin1_, t1_, lambda_, opts___Rule] /; fin1 < 1 := 
Module[{m, n, r, i}, n = Length[matrixA]; 
m = Drop[matrixA, {n}, {n}] /. {opts} /. Options[qInjectionE] /. f1 -> fin1; 
r = Last /@ AcuteInput[m, Flatten[{1, PadLeft[{0}, n - 2]}], t1, q]; 
Thread[Flatten[{qB[t1], Table[Subscript[q, i][t1], {i, 2, n - 6}], qURI[t1], 
qULI[t1], qLLI[t1], qFEC[t1], qSI[t1]}] -> Chop[ExpandAll[Flatten[int*r*Exp[N[-lambda]*t1]]], 
10^(-45)]]]; 

qInjectionChronicE[matrixA_, int_, fin1_, t1_, lambda_, opts___Rule] /; fin1 < 1:= 
Module[{a, b, c, e,  x1, x2, x3, t, t2}, a = qInjectionE1[matrixA, int, fin1, t2, N[lambda], opts]; 
{b, c} = {qURI[t2], qFEC[t2]} /. a; d = Plus @@ Last /@ a; e = d - b - c; 
{x1[t_], x2[t_], x3[t_]} = {First /@ a, Last /@ a, e} /. t2 -> t; 
Thread[Flatten[{x1[t1], qWB[t1]}] -> Flatten[(qContinuous[1, {#1}, t, t1] & ) /@ 
 Flatten[{x2[t], x3[t]}]]]]; 

qInjectionChronicE[matrixA_, int_, fin1_, t1_, lambda_, opts___]/; fin1 >= 1 := "f1 must be lower than 1";

qInjectionContinuousE[matrixA_, {int_, t3_}, fin1_, t1_, lambda_, opts___Rule] /; fin1 < 1:= 
Module[{a, b, c, e,  x1, x2, x3, t, t2}, a = qInjectionE1[matrixA, 1, fin1, t2, N[lambda], opts]; 
{b, c} = {qURI[t2], qFEC[t2]} /. a; d = Plus @@ Last /@ a; e = d - b - c; 
{x1[t_], x2[t_], x3[t_]} = {First /@ a, Last /@ a, e} /. t2 -> t; 
Thread[Flatten[{x1[t1], qWB[t1]}] -> Flatten[(qContinuous[int, {#1}, t3, t1] & ) /@ 
Flatten[{x2[t3], x3[t3]}]]]]; 

qInjectionContinuousE[matrixA_, int_, fin1_, t1_, lambda_, opts___]/; fin1 >= 1 := "f1 must be lower than 1";

qInjectionE["Acute",matrixA_, int_, fin1_, t1_, lambda_, opts___Rule] := 
  qInjectionE1[matrixA, int, fin1, t1, lambda, opts]  

qInjectionE["Constant",matrixA_, int_, fin1_, t1_, lambda_, opts___Rule] := 
     qInjectionChronicE[matrixA, int, fin1, t1,  lambda, opts]; 

qInjectionE["Continuous",matrixA_, {int_, t3_}, fin1_, t1_, lambda_, opts___Rule] := 
     qInjectionContinuousE[matrixA, {int, t3}, fin1, t1, lambda, opts]; 

qInjectionE[matrixA_, int_, fin1_, t1_,  lambda_, opts___Rule] := 
   Module[{meth}, meth = IntakeType /. {opts} /. Options[qInjectionE]; 
     qInjectionE[meth, matrixA, int, fin1, t1, lambda, opts]]/; fin1 < 1; 

qInjectionE[matrixA_, int_, fin1_, t1_, lambda_, opts___]/; fin1 >= 1 := "f1 must be lower than 1";

qInjectionE[(matrixA_)?MatrixQ, int_,  fin1_, t1_, lambda_, {(opts___)?OptionQ}]:=
"Something is wrong"; 

qWholebody[matrixA_, inp_, t1_, lambda_, opts___Rule] := 
Module[{a, b, c, d}, a = qInjection1[matrixA, inp, t1, 0, opts]; 
{b, c} = {qURI[t1], qFEC[t1]} /. a; d = Plus @@ Last /@ a; 
Chop[ExpandAll[(d - b - c)*Exp[N[-lambda]*t1]], 10^(-45)]];

qWholebodyE[matrixA_, inp_, fin1_, t1_, lambda_, opts___Rule] := 
With[{m = {IntakeWay} /. Flatten[{opts}] /. Options[qInjectionE]},
Module[{a, b, c, d}, a = qInjectionE1[matrixA, inp, fin1, t1, 0, opts]; 
{b, c} = {qURI[t1], qFEC[t1]} /. a; d = Plus @@ Last /@ a; 
Chop[ExpandAll[(d - b - c)*Exp[N[-lambda]*t1]], 10^(-45)]]/; 
m === {"Injection"}];

(*End new en v 3.0*)

(* INGESTION*)

qGI[int_, f1_, t1_, lambda_, opts___] /; f1 > 1 := "f1 must be less than or iqual to 1"

qGI[int_, f1_, t1_, lambda_, opts___] /; f1 == 1 := 
Module[{gi}, gi = {Exp[(-Subscript[k, ST])*t1], 0, 1 - Exp[(-Subscript[k, ST])*t1], 0, 0, 
0} /. {opts} /. Options[qGI]; Thread[{qST[t1], qSI[t1], qBfromGI[t1], qULI[t1], 
qLLI[t1], qFEC[t1]} -> ExpandAll[int*gi*Exp[N[-lambda]*t1]]]]; 

qGI[int_, f1_, t1_, lambda_, opts___]/;f1<1 := 
Module[{ic, icrpGIModel, gi}, ic = {1, 0, 0, 0, 0, 0}; 
icrpGIModel = CompartMatrix[6, {{1, 2, Subscript[k, ST]}, 
{2, 3, (f1*Subscript[k, SI])/(1 - f1)}, {2, 4, Subscript[k, SI]}, 
{4, 5, Subscript[k, ULI]}, {5, 6, Subscript[k, LLI]}}] /. {opts} /. Options[qGI]; 
gi = Chop[ExpandAll[Last /@ AcuteInput[icrpGIModel, ic, t1, q]]]; 
Thread[{qST[t1],qSI[t1],  qBfromGI[t1], qULI[t1], qLLI[t1], 
qFEC[t1]} -> ExpandAll[int*gi*Exp[N[-lambda]*t1]]]/. (x_)?NumberQ :> N[x]]; 

(* New in version 3.0*)

qIngestion1[matrixA_, int_, f1_,  t1_, lambda_, opts___] := 
Module[{t, n, m, qst, qsi, acumultedB, uli, lli, acumultedfec, qgi, inputB, total, i}, 
n = Length[matrixA]; m = matrixA /. {opts} /. Options[qGI]; 
qgi = qGI[1, f1, t, 0, opts]; 
{qst, qsi, acumultedB, uli, lli, acumultedfec} = 
{qST[t], qSI[t], qBfromGI[t], qULI[t], qLLI[t], qFEC[t]} /. qgi; 
inputB = D[acumultedB, t]; 
total = {qst, qsi, Last /@ MultiExpInput[m, PadLeft[{}, n], 
PadRight[{inputB}, n], t, t, x] + PadLeft[{uli, lli, acumultedfec}, n]} /. t -> t1; 
Thread[Flatten[{qST[t1], qSI[t1], qB[t1], Table[Subscript[q, i][t1], {i, 2, n - 4}], 
qURI[t1], qULI[t1], qLLI[t1], qFEC[t1]}] -> Flatten[ExpandAll[int*total*Exp[N[-lambda]*t1]]]]]

qIngestionChronic[matrix_, int_, fin1_, t1_, lambda_, options___Rule] /; 
fin1 <= 1 := Module[{a, b, c, d, e, x1, x2, x3, w2, t, t2}, 
a = qIngestion1[matrix, int, fin1, t2, lambda, options]; 
{b, c} = {qURI[t2], qFEC[t2]} /. a; d = Plus @@ Last /@ a; 
e = d - b - c; {x1[t_], x2[t_], x3[t_]} = {First /@ a, Last /@ a, e} /. 
t2 -> t; Thread[Flatten[{x1[t1], qWB[t1]}] -> 
Flatten[(qContinuous[1, {#1}, t, t1] & ) /@ Flatten[{x2[t], x3[t]}]]]]; 

qIngestionChronic[matrix_, int_, fin1_, t1_, lambda_, options___]/; fin1 > 1 := 
"f1 must be less than or equal to 1" ;

qIngestionContinuous[matrix_, {int_, t3_}, fin1_, t1_, lambda_, options___Rule] /; 
fin1 <= 1 := Module[{a, b, c, d, e,  x1, x2, x3, w2, t, t2}, 
a = qIngestion1[matrix, 1, fin1, t2, lambda, options]; 
{b, c} = {qURI[t2], qFEC[t2]} /. a; d = Plus @@ Last /@ a; 
e = d - b - c; {x1[t_], x2[t_], x3[t_]} = {First /@ a, Last /@ a, e} /. 
t2 -> t; Thread[Flatten[{x1[t1], qWB[t1]}] -> 
Flatten[(qContinuous[int, {#1}, t3, t1] & ) /@ Flatten[{x2[t3], x3[t3]}]]]]; 

qIngestionContinuous[matrix_, int_, fin1_, t1_, lambda_, options___]/; fin1 > 1 := 
"f1 must be iqual to or lower than 1";

qIngestion["Acute",matrixA_, int_, fin1_, t1_, lambda_, opts___Rule] := 
  qIngestion1[matrixA, int, fin1, t1, lambda, opts]  

qIngestion["Constant",matrixA_, int_, fin1_, t1_, lambda_, opts___Rule] := 
     qIngestionChronic[matrixA, int, fin1, t1,  lambda, opts]; 

qIngestion["Continuous",matrixA_, {int_, t3_}, fin1_, t1_, lambda_, opts___Rule] := 
     qIngestionContinuous[matrixA, {int, t3}, fin1, t1, lambda, opts]; 

qIngestion[matrixA_, int_, fin1_, t1_,  lambda_, opts___Rule] := 
Module[{meth}, meth = IntakeType /. {opts} /. Options[qIngestion]; 
qIngestion[meth, matrixA, int, fin1, t1, lambda, opts]]/; fin1 <= 1; 

qIngestion[matrix_, int_, fin1_, t1_, lambda_, options___]/; fin1 > 1 := 
"f1 must be less than or equal to 1" ;

qWholebody[matrixA_, inp_,fin1_, t1_, lambda_, opts___Rule] := 
Module[{a, b, c, d}, a = qIngestion1[matrixA, inp, fin1, t1, 0, opts]; 
{b, c} = {qURI[t1], qFEC[t1]} /. a; d = Plus @@ Last /@ a; 
Chop[ExpandAll[(d - b - c)*Exp[N[-lambda]*t1]], 10^(-45)]];

qIngestionE1[matrixA_, int_, fin1_, t1_, lambda_, opts___]:= 
Module[{t, m, n, r, i}, n = Length[matrixA]; 
m = matrixA /.{opts} /. Options[qGI] /. f1 -> fin1; 
r = Last /@ AcuteInput[m, Flatten[{PadLeft[{0}, n - 1], 1}], t1, q]; 
Thread[Flatten[{qB[t1], Table[Subscript[q, i][t1], {i, 2, n - 6}], qURI[t1], qULI[t1], 
qLLI[t1], qFEC[t1], qSI[t1], qST[t1]}] -> Flatten[ExpandAll[int*r*Exp[N[-lambda]*t1]]]]]/; fin1 <1;

qIngestionChronicE[matrix_, int_, fin1_, t1_, lambda_, options___Rule] /; 
fin1 <= 1 := Module[{a, b, c, d, e,  x1, x2, x3, w2, t, t2}, 
a = qIngestionE1[matrix, int, fin1, t2, lambda, options]; 
{b, c} = {qURI[t2], qFEC[t2]} /. a; d = Plus @@ Last /@ a; 
e = d - b - c; {x1[t_], x2[t_], x3[t_]} = {First /@ a, Last /@ a, e} /. 
t2 -> t; Thread[Flatten[{x1[t1], qWB[t1]}] -> 
Flatten[(qContinuous[1, {#1}, t, t1] & ) /@ Flatten[{x2[t], x3[t]}]]]]; 

qIngestionChronicE[matrix_, int_, fin1_, t1_, lambda_, options___]/; fin1 >= 1 := 
"f1 must be less than 1";

qIngestionContinuousE[matrix_,{int_,t3_},fin1_,t1_,lambda_,options___Rule]/;fin1<1:=
Module[{a,b,c,d,e,x1,x2,x3,w2,t,t2},a=qIngestionE1[matrix,1,fin1,t2,lambda,options];
{b,c}={qURI[t2],qFEC[t2]}/.a;d=Plus@@Last/@a;e=d-b-c;{x1[t_],x2[t_],x3[t_]}={First/@a,Last/@a,e}/.t2->t;
Thread[Flatten[{x1[t1],qWB[t1]}]-> Flatten[(qContinuous[int,{#1},t3,t1]&)/@Flatten[{x2[t3],x3[t3]}]]]];
 
qIngestionContinuousE[matrix_, int_, fin1_, t1_, lambda_, options___]/; fin1 >= 1 := 
"f1 must be less than 1";

qIngestionE["Acute",matrixA_, int_, fin1_, t1_, lambda_, opts___Rule] := 
  qIngestionE1[matrixA, int, fin1, t1, lambda, opts]  

qIngestionE["Constant",matrixA_, int_, fin1_, t1_, lambda_, opts___Rule] := 
     qIngestionChronicE[matrixA, int, fin1, t1,  lambda, opts]; 

qIngestionE["Continuous",matrixA_, {int_, t3_}, fin1_, t1_, lambda_, opts___Rule] := 
     qIngestionContinuousE[matrixA, {int, t3}, fin1, t1, lambda, opts]

qIngestionE[matrixA_, int_, fin1_, t1_,  lambda_, opts___Rule] := 
Module[{meth}, meth = IntakeType /. {opts} /. Options[qIngestionE]; 
 qIngestionE[meth, matrixA, int, fin1, t1, lambda, opts]]/; fin1 <= 1 ; 

qIngestionE[matrixA_, int_, fin1_, t1_, lambda_, opts___]:= "f1 must be less than 1" 

qWholebodyE[matrixA_, inp_,fin1_, t1_, lambda_, opts___Rule] := 
With[{m = {IntakeWay} /. Flatten[{opts}] /. Options[qInjectionE]},
Module[{a, b, c, d}, a = qIngestionE1[matrixA, inp, fin1, t1, 0, opts]; 
{b, c} = {qURI[t1], qFEC[t1]} /. a; d = Plus @@ Last /@ a; 
Chop[ExpandAll[(d - b - c)*Exp[N[-lambda]*t1]], 10^(-45)]]/; 
m === {"Ingestion"}];

(*End new en v 3.0*)

(* INHALATION*)

(*qRT[int_, {AIreg_, bbfastseqreg_, bbslowreg_, 
     BBfastseqreg_, BBslowreg_, ET2reg_, ET1reg_}, 
    {sp_, spt_, ss_}, t1_, lambda_?NumericQ, 
    opts___] := Module[{t, ic, icrp66model, 
     initialcondition, icrp66modelTransformed, 
     respiratoryTract, respiratoriTractTransformed, 
     totalRT, qET1ret, i}, 
    ic = {AI1, AI2, AI3, bb1, bb2, bbseq, BB1, BB2, 
          BBseq, 0, ET2, ETseq, 0, 0, 0} /. {opts} /. 
        Options[qRT] /. {AI -> AIreg, 
        bb -> bbfastseqreg + bbslowreg, 
        BB -> BBfastseqreg + BBslowreg, 
        fbs -> bbslowreg/(bbfastseqreg + bbslowreg), 
        fBs -> BBslowreg/(BBfastseqreg + BBslowreg), 
        ET2 -> ET2reg}; 
      icrp66model = CompartMatrix[15, {{1, 4, Subscript[k, "AI1,bb1"]}, 
      {2, 4, Subscript[k, "AI2,bb1"]}, {3, 4, Subscript[k, "AI3,bb1"]}, 
      {3, 10, Subscript[k, "AI3,TH"]}, {4, 7, Subscript[k, "bb1,BB1"]}, 
      {5, 7, Subscript[k, "bb2,BB1"]}, {6, 10, Subscript[k, "bbseq,TH"]}, 
      {7, 11, Subscript[k, "BB1,ET2"]}, {8, 11, Subscript[k, "BB2,ET2"]}, 
      {9, 10, Subscript[k, "BBseq,TH"]}, {10, 14, sp}, 
      {11, 15, Subscript[k, "ET2,ST"]}, {12, 13, Subscript[k, "ETseq,ET"]}, 
      {13, 14, sp}, {1, 0, spt}, {2, 0, spt}, {3, 0, spt}, {4, 0, spt}, {5, 0, spt}, 
      {6, 0, spt}, {7, 0, spt}, {8, 0, spt}, 
      {9, 0, spt}, {10, 0, spt}, {11, 0, spt}, 
      {12, 0, spt}, {13, 0, spt}, {1, 14, sp}, 
      {2, 14, sp}, {3, 14, sp}, {4, 14, sp}, 
      {5, 14, sp}, {6, 14, sp}, {7, 14, sp}, 
      {8, 14, sp}, {9, 14, sp}, {11, 14, sp}, 
      {12, 14, sp}, {13, 14, sp}}, lambda] /. {opts} /. Options[qRT]; 
     icrp66modelTransformed = 
      CompartMatrix[15, {{1, 4, Subscript[k, "AI1,bb1"]}, {2, 4, Subscript[k, "AI2,bb1"]}, 
      {3, 4, Subscript[k, "AI3,bb1"]}, {3, 10, Subscript[k, "AI3,TH"]}, 
      {4, 7, Subscript[k, "bb1,BB1"]}, {5, 7, Subscript[k, "bb2,BB1"]}, 
      {6, 10, Subscript[k, "bbseq,TH"]}, {7, 11, Subscript[k, "BB1,ET2"]}, 
      {8, 11, Subscript[k, "BB2,ET2"]}, {9, 10, Subscript[k, "BBseq,TH"]}, {10, 14, ss}, 
      {11, 15, Subscript[k, "ET2,ST"]}, {12, 13, Subscript[k, "ETseq,ET"]}, 
      {1, 14, ss}, {2, 14, ss}, {3, 14, ss}, {4, 14, ss}, {5, 14, ss}, {6, 14, ss}, 
      {7, 14, ss}, {8, 14, ss}, {9, 14, ss}, {10, 14, ss}, {11, 14, ss}, {12, 14, ss}, 
      {13, 14, ss}}, lambda] /. {opts} /. Options[qRT]; 
      respiratoryTract = Chop[ExpandAll[Last /@ AcuteInput[icrp66model, ic, t, q]]]; 
      initialcondition = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; 
     respiratoriTractTransformed = Last /@ MultiExpInput[icrp66modelTransformed, initialcondition, 
     Flatten[{spt*Drop[respiratoryTract, -2], 0, 0}], t, t, q]; 
     totalRT = int*Chop[ExpandAll[respiratoryTract + respiratoriTractTransformed]] /. t -> t1; 
     qET1ret = int*ET1reg*Exp[(-(lambda + Subscript[k, Env]))*t1] /. {opts} /. Options[qRT] ; 
     Flatten[{qET1[t1] -> qET1ret, Thread[Flatten[{Table[Subscript[q, i][t1], {i, 1, 13}], 
     {qBfromRT[t1], qST[t1]}}] -> totalRT],  qLungs[t1] -> Plus @@ Drop[totalRT, -5]}] 
     /. {(x_)?IntegerQ :> x, (x_)?NumberQ :> N[x]}]; *)


(*new en v 3.1*)

qRT[int_, {AIreg_, bbfastseqreg_, bbslowreg_, BBfastseqreg_, BBslowreg_, 
ET2reg_, ET1reg_}, {sp_, spt_, ss_}, t1_, (lambda_)?NumericQ, 
opts___] := Module[{a, qet1,  ai, AI1o, AI2o, AI3o, bb1o,bb2o,  bbseqo, 
BB1o, BB2o, BBseqo, ET2o, ETseqo, t, ic, icrp66model, initialcondition, 
icrp66modelTransformed, respiratoryTract, respiratoriTractTransformed, 
totalRT, qET1ret, i}, ai = AIreg; {AI1o = fAI1*ai, AI2o = fAI2*ai, AI3o = fAI3*ai, 
bb1o = (1 - fbs)*bbfastseqreg - fbs*bbslowreg, bb2o = bbslowreg, 
bbseqo = fbs*(bbfastseqreg + bbslowreg), BB1o = (1 - fBs)*BBfastseqreg - fBs*BBslowreg, 
BB2o = BBslowreg, BBseqo = fBs*(BBfastseqreg + BBslowreg), ET2o = fet2*ET2reg, 
ETseqo = (1 - fet2)*ET2reg} ;ic = {AI1o, AI2o, AI3o, bb1o, bb2o, bbseqo, BB1o, 
BB2o, BBseqo, 0, ET2o, ETseqo, 0, 0, 0}/. {opts} /. Options[qRT]; 
icrp66model = CompartMatrix[15, {{1, 4, Subscript[k, "AI1,bb1"]}, 
          {2, 4, Subscript[k, "AI2,bb1"]}, {3, 4, Subscript[k, "AI3,bb1"]}, 
          {3, 10, Subscript[k, "AI3,TH"]}, {4, 7, Subscript[k, "bb1,BB1"]}, 
          {5, 7, Subscript[k, "bb2,BB1"]}, {6, 10, Subscript[k, "bbseq,TH"]}, 
          {7, 11, Subscript[k, "BB1,ET2"]}, {8, 11, Subscript[k, "BB2,ET2"]}, 
          {9, 10, Subscript[k, "BBseq,TH"]}, {10, 14, sp}, 
          {11, 15, Subscript[k, "ET2,ST"]}, {12, 13, Subscript[k, 
            "ETseq,ET"]}, {13, 14, sp}, {1, 0, spt}, {2, 0, spt}, 
          {3, 0, spt}, {4, 0, spt}, {5, 0, spt}, {6, 0, spt}, {7, 0, spt}, 
          {8, 0, spt}, {9, 0, spt}, {10, 0, spt}, {11, 0, spt}, {12, 0, spt}, 
          {13, 0, spt}, {1, 14, sp}, {2, 14, sp}, {3, 14, sp}, {4, 14, sp}, 
          {5, 14, sp}, {6, 14, sp}, {7, 14, sp}, {8, 14, sp}, {9, 14, sp}, 
          {11, 14, sp}, {12, 14, sp}, {13, 14, sp}}, lambda] /. {opts} /. 
       Options[qRT]; icrp66modelTransformed = 
      CompartMatrix[15, {{1, 4, Subscript[k, "AI1,bb1"]}, 
          {2, 4, Subscript[k, "AI2,bb1"]}, {3, 4, Subscript[k, "AI3,bb1"]}, 
          {3, 10, Subscript[k, "AI3,TH"]}, {4, 7, Subscript[k, "bb1,BB1"]}, 
          {5, 7, Subscript[k, "bb2,BB1"]}, {6, 10, Subscript[k, "bbseq,TH"]}, 
          {7, 11, Subscript[k, "BB1,ET2"]}, {8, 11, Subscript[k, "BB2,ET2"]}, 
          {9, 10, Subscript[k, "BBseq,TH"]}, {10, 14, ss}, 
          {11, 15, Subscript[k, "ET2,ST"]}, {12, 13, Subscript[k, 
            "ETseq,ET"]}, {1, 14, ss}, {2, 14, ss}, {3, 14, ss}, {4, 14, ss}, 
          {5, 14, ss}, {6, 14, ss}, {7, 14, ss}, {8, 14, ss}, {9, 14, ss}, 
          {10, 14, ss}, {11, 14, ss}, {12, 14, ss}, {13, 14, ss}}, lambda] /. 
        {opts} /. Options[qRT]; respiratoryTract = 
      Chop[ExpandAll[Last /@ AcuteInput[icrp66model, ic, t, q]]]; 
     initialcondition = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; 
     respiratoriTractTransformed = Last /@ MultiExpInput[
        icrp66modelTransformed, initialcondition, 
        Flatten[{spt*Drop[respiratoryTract, -2], 0, 0}], t, t, q]; 
     totalRT = int*Chop[ExpandAll[respiratoryTract + 
           respiratoriTractTransformed]] /. t -> t1; 
     qET1ret = int*ET1reg*Exp[(-(lambda + Subscript[k, Env]))*t1] /. 
        {opts} /. Options[qRT]; 
     Flatten[{qET1[t1] -> qET1ret, Thread[
         Flatten[{Table[Subscript[q, i][t1], {i, 1, 13}], {qBfromRT[t1], 
             qST[t1]}}] -> totalRT], qLungs[t1] -> 
         Plus @@ Drop[totalRT, -5]}] /. {(x_)?IntegerQ :> x, 
       (x_)?NumberQ :> N[x]}]; 

(* Nota.- en qRTandGI no interviene lambda, no se incluye como parte del imput *)

qRTandGI[int_, ifd_, fra_, f1_, t1_, options___] /; f1 < 1 := 
 Module[{gimatrix, inputST, t}, 
 gimatrix = CompartMatrix[6, {{1, 2, Subscript[k, ST]}, {2, 3, (f1*Subscript[k, SI])/(1 - f1)}, 
 {2, 4, Subscript[k, SI]}, {4, 5, Subscript[k, ULI]}, {5, 6, Subscript[k, LLI]}}] /. 
 Options[qGI]; inputST = {D[qSTfromRT[int, ifd, fra, t, 0, options], t], 0, 0, 0, 0, 0}; 
 Thread[{qLungs[t1], qST[t1], qSI[t1], qB[t1], qULI[t1], qLLI[t1], qFEC[t1]} -> 
Flatten[{qLungs[int, ifd, fra, t1, 0, options],
 Last/@ MultiExpInput[gimatrix, {0, 0, 0, 0, 0, 0}, 
 inputST, t, t1, q] + {0, 0, qBfromRT[int, ifd, fra, t1, 0, options], 0, 0, 0}}]]]
 
 qRTandGI[int_, ifd_, fra_, f1_, t1_, options___] /; f1 == 1 := 
 Module[{gimatrix, inputST, t}, 
 gimatrix = CompartMatrix[6, {{1, 3, Subscript[k, ST]} }] /. 
 Options[qGI]; inputST = {D[qSTfromRT[int, ifd, fra, t, 0, options], t], 0, 0, 0, 0, 0}; 
 Thread[{qLungs[t1], qST[t1], qSI[t1], qB[t1], qULI[t1], qLLI[t1], qFEC[t1]} -> 
Flatten[{qLungs[int, ifd, fra, t1, 0, options],
 Last/@ MultiExpInput[gimatrix, {0, 0, 0, 0, 0, 0}, 
 inputST, t, t1, q] + {0, 0, qBfromRT[int, ifd, fra, t1, 0, options], 0, 0, 0}}]]];

qRTandGI[int_, {AIreg_, bbfastseqreg_, bbslowreg_, 
     BBfastseqreg_, BBslowreg_, ET2reg_, ET1reg_}, 
    {sp_, spt_, ss_}, f1_,  t1_,  opts___] /; f1 > 1 := 
  "f1 must be less than or equal to 1";



(* New in version 3.0, modified in 3.1*)

qInhalation1[matrixA_List, int_, ifd_List, fra_List, fin1_, t1_, lambda_, opts___] /; fin1 <= 1:= 
  Module[{t, n, m, qrtgi, qlungs, qst, qsi, acumultedB, uli, lli, acumultedfec, 
    inputB, total, i}, n = Length[matrixA]; m = matrixA /. {opts} /. Options[qGI]; 
    qrtgi = qRTandGI[1, ifd, fra, fin1, t, opts]; 
    {qlungs, qst, qsi, acumultedB, uli, lli, acumultedfec} = 
     {qLungs[t], qST[t], qSI[t], qB[t], qULI[t], qLLI[t], qFEC[t]} /. qrtgi; 
    inputB = D[acumultedB, t]; 
    total = {qlungs, qst, qsi, Last /@ MultiExpInput[m, PadLeft[{}, n], 
          PadRight[{inputB}, n], t, t, x] + PadLeft[{uli, lli, acumultedfec}, 
         n]} /. t -> t1; Thread[Flatten[{qLungs[t1], qST[t1], qSI[t1], qB[t1], 
        Table[Subscript[q, i][t1], {i, 2, n - 4}], qURI[t1], qULI[t1], qLLI[t1], 
        qFEC[t1]}] -> Flatten[ExpandAll[int*total*Exp[-N[lambda] t1]]]]];

qInhalation1[matrixA_, int_, ifd_, fra_, fin1_, t1_, lambda_, 
    opts___] /; fin1 > 1:=  "f1 must be less than or equal to 1";

qInhalationChronic[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, 
options___Rule] /; fin1 <= 1 := Module[{a, b, x1, g1, x2, g2, w, t, t2}, 
a = qRTChronic[int,ifd,fra,t2,lambda,options]; 
b = qInhalation1[matrix, 1, ifd, fra, fin1, t2, lambda, options];
{x1[t_], g1[t_]} = {First /@ a, Last /@ a} /. t2 -> t;
{x2[t_], g2[t_]} = {First /@ b, Last /@ b} /. t2 -> t; 
Thread[Flatten[{x1[t1],x2[t1]}] -> Flatten[{g1[t1],(qContinuous[int, {#1}, t, t1] & ) /@ g2[t]}]]]; 

qInhalationChronic[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, options___] /; fin1 > 1 := 
 "f1 must be less than or equal to 1";

qInhalation["Acute", matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, opts___Rule] :=
    Module[{a,b,x1,g1,x2,g2,w,t,t2},a=qRT2[int,ifd,fra,t2,lambda,opts];
      b=qInhalation1[matrix,int,ifd,fra,fin1,t2,lambda,opts];
      {x1[t_],g1[t_]}={First/@a,Last/@a}/.t2\[Rule]t;
      {x2[t_],g2[t_]}={First/@b,Last/@b}/.t2\[Rule]t;
      Thread[Flatten[{x1[t1],x2[t1]}]\[Rule]Flatten[{g1[t1],g2[t1]}]]];

qInhalation["Constant", matrix_, int_, ifd_, fra_, fin1_, t_, lambda_, opts___Rule] := 
qInhalationChronic[matrix, int, ifd, fra, fin1, t, lambda, opts];

qInhalation["Continuous", matrix_, int_, ifd_, fra_,fin1_,  t_, lambda_, opts___Rule] := 
"Option not available for Inhalation";

qInhalation[matrix_, int_, ifd_, fra_, fin1_, t_, lambda_, opts___Rule] := 
Module[{meth}, meth = IntakeType /. {opts} /. Options[qInhalation]; 
qInhalation[meth, matrix, int, ifd, fra, fin1, t, lambda, opts]]/;fin1<=1;

qInhalation[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, options___] /; fin1 >1 := 
 "f1 must be less than or equal to 1";


(*qInhalationE1[matrixA_, int_, ifd_, fra_, fin1_,  t1_, lambda_, 
    opts___]/; fin1 <1 := Module[{t, a, b, qlungs, m, n, r, i}, 
    {qlungs, a, b} = {qLungs[t], qBfromRT[t], qST[t]} /. 
       qRT[1, ifd, fra, t, 0, opts]; n = Length[matrixA]; 
     m = matrixA /. {opts} /. Options[qGI] /. f1 -> fin1; 
     r = Flatten[{qlungs /. t -> t1, Last /@ MultiExpInput[m, 
          PadLeft[{0}, n], Flatten[{D[a, t], PadLeft[{0}, n - 2], D[b, t]}], 
          t, t1, q]}]; Thread[Flatten[{qLungs[t1], qB[t1], 
         Table[Subscript[q, i][t1], {i, 2, n - 6}], qURI[t1], qULI[t1], 
         qLLI[t1], qFEC[t1], qSI[t1], qST[t1]}] -> 
       Flatten[ExpandAll[int*r*Exp[-N[lambda]*t1]]]]]; *)

qInhalationE1[matrixA_, int_, ifd_, fra_, fin1_,  t1_, lambda_, 
    opts___]/; fin1 <1 := Module[{t, t2, a, b, qlungs, m, n, r, i}, 
    {qlungs, a, b} = {qLungs[1, ifd, fra, t2, 0, opts], qBfromRT[1, ifd, fra, t, 0, opts], 
    qSTfromRT[1, ifd, fra, t, 0, opts]}/.t2->t; n = Length[matrixA]; 
     m = matrixA /. {opts} /. Options[qGI] /. f1 -> fin1; 
     r = Flatten[{qlungs /. t -> t1, Last /@ MultiExpInput[m, 
          PadLeft[{0}, n], Flatten[{D[a, t], PadLeft[{0}, n - 2], D[b, t]}], 
          t, t1, q]}]; Thread[Flatten[{qLungs[t1], qB[t1], 
         Table[Subscript[q, i][t1], {i, 2, n - 6}], qURI[t1], qULI[t1], 
         qLLI[t1], qFEC[t1], qSI[t1], qST[t1]}] -> 
       Flatten[ExpandAll[int*r*Exp[-N[lambda]*t1]]]]]; 

qInhalationE1[matrixA_, int_, ifd_, fra_, fin1_, t1_, lambda_, 
    opts___]/; fin1 >= 1 :=  "f1 must be less than 1";

qInhalationChronicE[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, 
options___Rule] /; fin1 <= 1 := Module[{a, b, x1, g1, x2, g2, w, t, t2}, 
a = qRTChronic[int,ifd,fra,t2,lambda,options]; 
b = qInhalationE1[matrix, 1, ifd, fra, fin1, t2, lambda, options];
{x1[t_], g1[t_]} = {First /@ a, Last /@ a} /. t2 -> t;
{x2[t_], g2[t_]} = {First /@ b, Last /@ b} /. t2 -> t; 
Thread[Flatten[{x1[t1],x2[t1]}] -> Flatten[{g1[t1],(qContinuous[int, {#1}, t, t1] & ) /@ g2[t]}]]]; 

qInhalationChronicE[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, options___]/; fin1 >= 1 := 
"f1 must be less than 1";

qInhalationE["Acute", matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, opts___Rule] :=
    Module[{a,b,x1,g1,x2,g2,w,t,t2},a=qRT2[int,ifd,fra,t2,lambda,opts];
      b=qInhalationE1[matrix,int,ifd,fra,fin1,t2,lambda,opts];
      {x1[t_],g1[t_]}={First/@a,Last/@a}/.t2\[Rule]t;
      {x2[t_],g2[t_]}={First/@b,Last/@b}/.t2\[Rule]t;
      Thread[Flatten[{x1[t1],x2[t1]}]\[Rule]Flatten[{g1[t1],g2[t1]}]]]/;fin1<1;

qInhalationE["Constant", matrix_,int_, ifd_, fra_, fin1_, t_, lambda_, opts___Rule] := 
qInhalationChronicE[matrix, int, ifd, fra, fin1, t, lambda, opts];

qInhalationE["Continuous",matrix_, int_, ifd_, fra_, fin1_, t_, lambda_, opts___Rule] := 
"Option not available for Inhalation";

qInhalationE[matrix_, int_, ifd_, fra_,  fin1_, t_, lambda_, opts___Rule] := 
Module[{meth}, meth = IntakeType /. {opts} /. Options[qInhalationE]; 
qInhalationE[meth, matrix, int, ifd, fra, fin1, t, lambda, opts]]/;fin1<1;

qInhalationE[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, options___] /; fin1 >= 1 := 
 "f1 must be less than 1";

qWholebody[matrixA_, inp_, ifd_, fra_, fin1_, t1_, lambda_, opts___Rule] /; fin1 <= 1 := 
Module[{a, b, c, d, e}, a = qInhalation1[matrixA, 1, ifd, fra, fin1, t1, 0, opts]; 
{b, c, d} = {qLungs[t1], qURI[t1], qFEC[t1]} /. a; e = Plus @@ Last /@ a; 
Chop[ExpandAll[inp*(e - b - c - d + Plus @@ Last /@ Drop[qRT[1, ifd, fra, t1, 0, opts], -3])*
 Exp[N[-lambda]*t1]], 10^(-45)]]; 

qWholebody[matrixA_, inp_, ifd_, fra_, fin1_, t1_, lambda_, opts___Rule]/; fin1 > 1 :=  
"f1 must be less than or equal to 1";

qWholebodyE[matrixA_, inp_, ifd_, fra_, fin1_, t1_, lambda_, opts___Rule] /; fin1 < 1 := 
Module[{a, b, c, d, e}, a = qInhalationE1[matrixA, 1, ifd, fra, fin1, t1, 0, opts]; 
{b, c, d} = {qLungs[t1], qURI[t1], qFEC[t1]} /. a; e = Plus @@ Last /@ a; 
Chop[ExpandAll[inp*(e - b - c - d + Plus @@ Last /@ Drop[qRT[1, ifd, fra, t1, 0, opts], -3])*
Exp[N[-lambda]*t1]], 10^(-45)]]; 

qWholebodyE[matrixA_, inp_, ifd_, fra_, fin1_, t1_, lambda_, opts___Rule]/; fin1 >= 1 :=  
"f1 must be less than 1";

(* BIOASSAY REPORTS*)

(*injection*)

BioassayReport[matrix_,int_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for MultiInputs intake"/;m==={"Injection","MultiInputs","CompartmentContent"}];

BioassayReportE[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for MultiInputs intake"/;m==={"Injection","MultiInputs","CompartmentContent"}];

BioassayReport[matrix_,int_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for continuous intake"/;m==={"Injection","Continuous","Disintegrations"}];

BioassayReportE[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for continuous intake"/;m==={"Injection","Continuous","Disintegrations"}];

BioassayReport[matrix_,int_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for MultiInputs intake"/;m==={"Injection","MultiInputs","Disintegrations"}];

BioassayReportE[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for MultiInputs intake"/;m==={"Injection","MultiInputs","Disintegrations"}];

(*Ingestion*)

BioassayReport[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for MultiInputs intake"/;m==={"Ingestion","MultiInputs","CompartmentContent"}];

BioassayReportE[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for MultiInputs intake"/;m==={"Ingestion","MultiInputs","CompartmentContent"}];

BioassayReport[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for continuous intake"/;m==={"Ingestion","Continuous","Disintegrations"}];

BioassayReportE[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for continuous intake"/;m==={"Ingestion","Continuous","Disintegrations"}];

BioassayReport[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for MultiInputs intake"/;m==={"Ingestion","MultiInputs","Disintegrations"}];

BioassayReportE[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for MultiInputs intake"/;m==={"Ingestion","MultiInputs","Disintegrations"}];

(*Inhalation*)

BioassayReport[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType}/.Flatten[{options}]/.Options[BioassayReport]},
"Option not available for inhalation"/;m==={"Inhalation","Continuous"}];

BioassayReportE[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType}/.Flatten[{options}]/.Options[BioassayReport]},
"Option not available for inhalation"/;m==={"Inhalation","Continuous"}];

BioassayReport[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]},
"Option not available for MultiInputs"/;m==={"Inhalation","MultiInputs","FullReport"}];

BioassayReportE[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]},
"Option not available for MultiInputs"/;m==={"Inhalation","MultiInputs","FullReport"}];

BioassayReport[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for MultiInputs intake"/;m==={"Inhalation","MultiInputs","CompartmentContent"}];

BioassayReportE[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for continuous intake"/;m==={"Inhalation","MultiInputs","CompartmentContent"}];

BioassayReport[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for MultiInputs intake"/;m==={"Inhalation","MultiInputs","Disintegrations"}];

BioassayReportE[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]}, 
"Option not available for continuous intake"/;m==={"Inhalation","MultiInputs","Disintegrations"}];

(* New in version 3.0*)

(*Injection*)

BioassayReport[matrix_, int_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport}/. Flatten[{options}] /. Options[BioassayReport]},
Module[{y, z, y0, z0, y1, z1, t, t2, a, b, q1, q2, q3, w, w1}, 
a = qInjection1[matrix, int, t,0, options];
{y, z} = {qURI[t], qFEC[t]} /. a; b = {y, z};
{q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}];
w = q3 - q1 - q2;
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = (y - y0)*Exp[N[-lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[N[-lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[N[-lambda]*t] /. t -> t2;
ExpandAll[{qDailyUrine[t1]-> y1[t1],qDailyFaecal[t1]->z1[t1], 
qWholebody[t1]-> w1[t1]}]] /;m === {"Injection", "Acute", "Automatic"}];

BioassayReport[matrix_, int_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport}/. Flatten[{options}] /. Options[BioassayReport]},
Module[{y, z, y0, z0, y1, z1, t, t2, a, b, q1, q2, q3, w, w1}, 
a = qInjection1[matrix, int, t,0, options];
{y, z} = {qURI[t], qFEC[t]} /. a; b = {y, z};
{q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}];
w = q3 - q1 - q2;
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = (y - y0)*Exp[N[-lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[N[-lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[N[-lambda]*t] /. t -> t2;
TableForm[Chop[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[{t1}, Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 10^-45], 
TableHeadings -> {None, {"Days after\nstart intake", "Urine\n24 hours", "Faecal\n24 hours", 
"Wholebody"}}]] /;m === {"Injection", "Acute", "BioassayTable"}];

BioassayReport[matrix_, int_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport}/. Flatten[{options}] /. Options[BioassayReport]},
Module[{y, z, y0, z0, y1, z1, t, t2, t3, a, b, q1, q2, q3, w, w1}, 
a = qInjection1[matrix, int, t,0, options];
{y, z} = {qURI[t], qFEC[t]} /. a; b = {y, z};
{q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}];
w = q3 - q1 - q2;
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = (y - y0)*Exp[N[-lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[N[-lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[N[-lambda]*t] /. t -> t2;
ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, t1}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Acute intake in t = 0", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]] /;m === {"Injection", "Acute", "GraphicReport"}];

BioassayReport[matrix_, int_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport}/. Flatten[{options}] /. Options[BioassayReport]},
Module[{y, z, y0, z0, y1, z1, t, t2, a, b, q1, q2, q3, w, w1}, 
a = qInjection1[matrix, int, t,0, options];
{y, z} = {qURI[t], qFEC[t]} /. a; b = {y, z};
{q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}];
w = q3 - q1 - q2;
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = (y - y0)*Exp[N[-lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[N[-lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[N[-lambda]*t] /. t -> t2;
{TableForm[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 
TableHeadings -> {None, {"Days after\nstart intake", "Urine\n24 hours", 
"Faecal\n24 hours", "Wholebody"}}], ExpandAll[{qDailyUrine[t1]-> y1[t1],
qDailyFaecal[t1]->z1[t1], qWholebody[t1]-> w1[t1]}],
ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, {t3, 1, 1000}, 
PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Acute intake in t = 0", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]}] /;m === {"Injection", "Acute", "FullReport"}];


BioassayReport[matrix_, int_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionChronic[matrix, int, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; ExpandAll[{qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1], 
qWholebody[t1] -> w1[t1]}]] /; m === {"Injection", "Constant", "Automatic"}]; 


BioassayReport[matrix_, int_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionChronic[matrix, int, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; TableForm[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[{t1}, Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 
TableHeadings -> {None, {"Days after\nstart intake", "Urine\n24 hours", "Faecal\n24 hours", 
"Wholebody"}}]] /; m === {"Injection", "Constant", "BioassayTable"}]; 

BioassayReport[matrix_, int_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2, t3}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionChronic[matrix, int, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, t1}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Constant intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]] /; m === {"Injection", "Constant", "GraphicReport"}]; 

BioassayReport[matrix_, int_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2, t3}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionChronic[matrix, int, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;  
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; {TableForm[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 
TableHeadings -> {None, {"Days after\nstart intake", "Urine\n24 hours", 
"Faecal\n24 hours", "Wholebody"}}], ExpandAll[{qDailyUrine[t1]-> y1[t1],
qDailyFaecal[t1]->z1[t1], qWholebody[t1]-> w1[t1]}],
ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, {t3, 1, 1000}, 
PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Constant intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]}] /; m === {"Injection", "Constant", "FullReport"}]; 

BioassayReport[matrix_, {int_, t4_}, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionContinuous[matrix, {int, t4}, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; ExpandAll[{qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1], 
qWholebody[t1] -> w1[t1]}]] /; m === {"Injection", "Continuous", "Automatic"}]; 

BioassayReport[matrix_, {int_, t4_}, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionContinuous[matrix, {int, t4}, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;  
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; TableForm[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[{t1}, Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 
TableHeadings -> {None, {"Days after\nstart intake", "Urine\n24 hours", "Faecal\n24 hours", 
"Wholebody"}}]] /; m === {"Injection", "Continuous", "BioassayTable"}]; 

BioassayReport[matrix_, {int_, t4_}, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionContinuous[matrix, {int,  t4}, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, t1}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Continuous intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]] /; m === {"Injection", "Continuous", "GraphicReport"}]; 

BioassayReport[matrix_, {int_, t4_}, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2, t3}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionContinuous[matrix, {int, t4}, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; {TableForm[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 
TableHeadings -> {None, {"Days after\nstart intake", "Urine\n24 hours", 
"Faecal\n24 hours", "Wholebody"}}], ExpandAll[{qDailyUrine[t1]-> y1[t1],
qDailyFaecal[t1]->z1[t1], qWholebody[t1]-> w1[t1]}],
ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, {t3, 1, 1000}, 
PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Constant intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]}] /; m === {"Injection", "Continuous", "FullReport"}]; 

BioassayReport[matrix_, int_, t1_, lambda_, options___Rule] := 
   With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. 
       Options[BioassayReport]}, qInjection1[matrix, int, t1, lambda, options] /; 
     m === {"Injection", "Acute", "CompartmentContent"}]; 
     
BioassayReport[matrix_, int_, t1_, lambda_, options___Rule] := 
   With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. 
       Options[BioassayReport]}, qInjectionChronic[matrix, int, t1, lambda, options] /; 
     m === {"Injection", "Constant", "CompartmentContent"}]; 

BioassayReport[matrix_, {int_, t_}, t1_, lambda_, options___Rule] := 
   With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. 
       Options[BioassayReport]}, qInjectionContinuous[matrix, {int, t}, t1, lambda, 
      options] /; m === {"Injection", "Continuous", "CompartmentContent"}]; 

BioassayReport[matrix_,int_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReport]}, qInjectionChronic[matrix,86400*int,t1,lambda,options]/;
        m==={"Injection","Acute","Disintegrations"}];

BioassayReport[matrix_,int_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReport]}, Module[{a, x1, x2},a =qInjectionChronic[matrix,86400*int,
    t2,lambda,options];{x1[t_],x2[t_]}={First/@a,Last/@a}/.t2\[Rule]t;
    Thread[x1[t1]\[Rule]Flatten [(qContinuous[1,{#1},t,t1]&)/@x2[t]]]]/;
        m==={"Injection","Constant","Disintegrations"}];

InjectionM[matrix_, int_List, t1_, lambda_, options___Rule] := 
   Module[{a, b, y, z, w, y0, z0, y1, z1, w1, t, t2, 
     list1, list2, list3}, 
    a = qInjection1[matrix, 1, t, lambda, options]; 
     {y, z} = {qURI[t], qFEC[t]} /. a; 
     b = Plus @@ Last /@ a; w = b - y - z; 
     y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
     y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; 
     z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
     w1[t2_] = w /. t -> t2; {list1, list2} = Transpose[Append[int, {0, t1}]]; 
     list3 = Reverse[t1 + list2[[-2]] - list2]; 
     ListConvolve[list1, (y1[#1] & ) /@ list3]; 
     ExpandAll[{ListConvolve[list1, (y1[#1] & ) /@ list3][[1]], 
     ListConvolve[list1, (z1[#1] & ) /@ list3][[1]], 
     ListConvolve[list1, (w1[#1] & ) /@ list3][[1]]}]]; 

BioassayReport[matrix_, int_List, t1_, lambda_, options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Thread[{qDailyUrine[t1], qDailyFaecal[t1], qWholebody[t1]} -> InjectionM[matrix, int, t1, lambda, options]]
 /; m === {"Injection", "MultiInputs", "Automatic"}]; 
 
BioassayReport[matrix_, int_List, t1_?IntegerQ, lambda_, options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]},
Module[ {x1,x2,x3, t, t2},{x1[t2_],x2[t2_],x3[t2_]}=InjectionM[matrix, int, t,lambda,options]/.t->t2 ; 
TableForm[Chop[{#,x1[#],x2[#],x3[#]}&/@Sort[Join[{t1},Range[10],Range[20,100,10],Range[200,1000,100],
Range[2000,10000,1000]]], 10^-45],TableHeadings\[Rule]{None,{"Days after\nlast intake","Urine\n24 hours",
"Faecal\n24 hours","Wholebody"}}]]/;m==={"Injection","MultiInputs","BioassayTable"}]; 

BioassayReport[matrix_, int_List, t1_, lambda_, options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]},
Module[ {x1,x2,x3, t, t2, t3},{x1[t2_],x2[t2_],x3[t2_]}=InjectionM[matrix, int, t,lambda,options]/.t->t2 ; 
ShowLegend[LogLogPlot[{x1[t3],x2[t3]},{t3,1,t1},PlotRange\[Rule]All,
PlotStyle\[Rule]styles,AxesLabel\[Rule]{"days","Bq"},
PlotLabel\[Rule]"Multi Inputs",ImageSize\[Rule]600,DisplayFunction\[Rule]Identity],{Reverse[
MapThread[{Graphics[{Sequence@@#1,Line[{{0,0},{1,0}}]}],#2}&,{styles,{"Urine\nExcretion",
"Faecal\nExcretion"}}]],LegendPosition\[Rule]{1,-0.3},LegendSize\[Rule]{0.5,0.5},LegendShadow\[Rule]{0.02,-0.02},
LegendSpacing\[Rule]0.2}]]/;m==={"Injection","MultiInputs","GraphicReport"}]; 


BioassayReportE[matrix_, int_, fin1_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport}/. Flatten[{options}] /. Options[BioassayReportE]},
Module[{y, z, y0, z0, y1, z1, t, t2, a, b, q1, q2, q3, w, w1}, 
a = qInjectionE1[matrix, int, fin1, t,0, options];
{y, z} = {qURI[t], qFEC[t]} /. a; b = {y, z};
{q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}];
w = q3 - q1 - q2;
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = (y - y0)*Exp[N[-lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[N[-lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[N[-lambda]*t] /. t -> t2;
ExpandAll[{qDailyUrine[t1]-> y1[t1],qDailyFaecal[t1]->z1[t1], 
qWholebody[t1]-> w1[t1]}]] /;m === {"Injection", "Acute", "Automatic"}];

BioassayReportE[matrix_, int_, fin1_, t1_?IntegerQ,  lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport}/. Flatten[{options}] /. Options[BioassayReportE]},
Module[{y, z, y0, z0, y1, z1, t, t2, a, b, q1, q2, q3, w, w1}, 
a = qInjectionE1[matrix, int, fin1, t,  0, options];
{y, z} = {qURI[t], qFEC[t]} /. a; b = {y, z};
{q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}];
w = q3 - q1 - q2;
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = (y - y0)*Exp[N[-lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[N[-lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[N[-lambda]*t] /. t -> t2;
TableForm[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[{t1}, Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 
TableHeadings -> {None, {"Days after\nstart intake", "Urine\n24 hours", "Faecal\n24 hours", 
"Wholebody"}}]] /;m === {"Injection", "Acute", "BioassayTable"}];

BioassayReportE[matrix_, int_, fin1_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport}/. Flatten[{options}] /. Options[BioassayReportE]},
Module[{y, z, y0, z0, y1, z1, t, t2, t3, a, b, q1, q2, q3, w, w1}, 
a = qInjectionE1[matrix, int, fin1, t,  0, options];
{y, z} = {qURI[t], qFEC[t]} /. a; b = {y, z};
{q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}];
w = q3 - q1 - q2;
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = (y - y0)*Exp[N[-lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[N[-lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[N[-lambda]*t] /. t -> t2;
ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, t1}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Acute intake in t = 0", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]] /;m === {"Injection", "Acute", "GraphicReport"}];

BioassayReportE[matrix_, int_, fin1_, t1_,  lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport}/. Flatten[{options}] /. Options[BioassayReportE]},
Module[{y, z, y0, z0, y1, z1, t, t2, a, b, q1, q2, q3, w, w1}, 
a = qInjectionE1[matrix, int, fin1, t, 0, options];
{y, z} = {qURI[t], qFEC[t]} /. a; b = {y, z};
{q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}];
w = q3 - q1 - q2;
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = (y - y0)*Exp[N[-lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[N[-lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[N[-lambda]*t] /. t -> t2;
{TableForm[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 
TableHeadings -> {None, {"Days after\nstart intake", "Urine\n24 hours", 
"Faecal\n24 hours", "Wholebody"}}], ExpandAll[{qDailyUrine[t1]-> y1[t1],
qDailyFaecal[t1]->z1[t1], qWholebody[t1]-> w1[t1]}],
ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, {t3, 1, 1000}, 
PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Acute intake in t = 0", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]}] /;m === {"Injection", "Acute", "FullReport"}];


BioassayReportE[matrix_, int_, fin1_, t1_,  lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionChronicE[matrix, int, fin1, t,  lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; ExpandAll[{qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1], 
qWholebody[t1] -> w1[t1]}]] /; m === {"Injection", "Constant", "Automatic"}]; 

BioassayReportE[matrix_, int_, fin1_, t1_?IntegerQ,  lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionChronicE[matrix, int, fin1, t,  lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; TableForm[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[{t1}, Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 
TableHeadings -> {None, {"Days after\nstart intake", "Urine\n24 hours", "Faecal\n24 hours", 
"Wholebody"}}]] /; m === {"Injection", "Constant", "BioassayTable"}]; 

BioassayReportE[matrix_, int_, fin1_, t1_?IntegerQ,  lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2, t3}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionChronicE[matrix, int, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, t1}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Constant intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]] /; m === {"Injection", "Constant", "GraphicReport"}]; 

BioassayReportE[matrix_, int_, fin1_, t1_,  lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2, t3}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionChronicE[matrix, int, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; {TableForm[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 
TableHeadings -> {None, {"Days after\nstart intake", "Urine\n24 hours", 
"Faecal\n24 hours", "Wholebody"}}], ExpandAll[{qDailyUrine[t1]-> y1[t1],
qDailyFaecal[t1]->z1[t1], qWholebody[t1]-> w1[t1]}],
ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, {t3, 1, 1000}, 
PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Constant intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]}] /; m === {"Injection", "Constant", "FullReport"}]; 

BioassayReportE[matrix_, {int_, t4_}, fin1_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionContinuousE[matrix, {int, t4},fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; ExpandAll[{qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1], 
qWholebody[t1] -> w1[t1]}]] /; m === {"Injection", "Continuous", "Automatic"}]; 

BioassayReportE[matrix_, {int_, t4_}, fin1_,  t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionContinuousE[matrix, {int, t4}, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;  
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; TableForm[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[{t1}, Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 
TableHeadings -> {None, {"Days after\nstart intake", "Urine\n24 hours", "Faecal\n24 hours", 
"Wholebody"}}]] /; m === {"Injection", "Continuous", "BioassayTable"}]; 

BioassayReportE[matrix_, {int_, t4_}, fin1_,t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionContinuousE[matrix, {int,  t4}, fin1,t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, t1}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Continuous intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]] /; m === {"Injection", "Continuous", "GraphicReport"}]; 

BioassayReportE[matrix_, {int_, t4_}, t1_, fin1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2, t3}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qInjectionContinuousE[matrix, {int, t4}, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
w1[t2_] = w /. t -> t2; {TableForm[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 
TableHeadings -> {None, {"Days after\nstart intake", "Urine\n24 hours", 
"Faecal\n24 hours", "Wholebody"}}], ExpandAll[{qDailyUrine[t1]-> y1[t1],
qDailyFaecal[t1]->z1[t1], qWholebody[t1]-> w1[t1]}],
ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, {t3, 1, 1000}, 
PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Continuous intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]}] /; m === {"Injection", "Continuous", "FullReport"}]; 

BioassayReportE[matrix_, int_, fin1_, t1_, lambda_, options___Rule] := 
   With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. 
       Options[BioassayReportE]}, qInjectionE[matrix, int, fin1, t1, lambda, options] /; 
     m === {"Injection", "Acute", "CompartmentContent"}]; 
     
BioassayReportE[matrix_, int_, fin1_, t1_, lambda_, options___Rule] := 
   With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. 
       Options[BioassayReportE]}, qInjectionChronicE[matrix, int, fin1, t1, lambda, options] /; 
     m === {"Injection", "Constant", "CompartmentContent"}]; 

BioassayReportE[matrix_, {int_, t_}, fin1_, t1_, lambda_, options___Rule] := 
   With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. 
       Options[BioassayReportE]}, qInjectionContinuousE[matrix, {int, t}, fin1,t1, lambda, 
      options] /; m === {"Injection", "Continuous", "CompartmentContent"}]; 

BioassayReportE[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReportE]}, qInjectionChronicE[matrix,86400*int,fin1,t1,lambda,options]/;
        m==={"Injection","Acute","Disintegrations"}];

BioassayReportE[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReportE]}, Module[{a, x1, x2},a =qInjectionChronicE[matrix,86400*int,
    fin1,t2,lambda,options];{x1[t_],x2[t_]}={First/@a,Last/@a}/.t2\[Rule]t;
    Thread[x1[t1]\[Rule]Flatten [(qContinuous[1,{#1},t,t1]&)/@x2[t]]]]/;
        m==={"Injection","Constant","Disintegrations"}];

InjectionME[matrix_, int_List, fin1_, t1_, lambda_, options___Rule] := 
   Module[{a, b, y, z, w, y0, z0, y1, z1, w1, t, t2, 
     list1, list2, list3}, 
    a = qInjectionE1[matrix, 1, fin1, t, lambda, options]; 
     {y, z} = {qURI[t], qFEC[t]} /. a; 
     b = Plus @@ Last /@ a; w = b - y - z; 
     y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
     y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; 
     z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
     w1[t2_] = w /. t -> t2; {list1, list2} = Transpose[Append[int, {0, t1}]]; 
     list3 = Reverse[t1 + list2[[-2]] - list2]; 
     ListConvolve[list1, (y1[#1] & ) /@ list3]; 
     ExpandAll[{ListConvolve[list1, (y1[#1] & ) /@ list3][[1]], 
     ListConvolve[list1, (z1[#1] & ) /@ list3][[1]], 
     ListConvolve[list1, (w1[#1] & ) /@ list3][[1]]}]]; 

BioassayReportE[matrix_, int_List, fin1_, t1_, lambda_, options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Thread[{qDailyUrine[t1], qDailyFaecal[t1], qWholebody[t1]} -> InjectionME[matrix, int, fin1, t1, lambda, options]]
 /; m === {"Injection", "MultiInputs", "Automatic"}]; 
 
BioassayReportE[matrix_, int_List, fin1_, t1_?IntegerQ, lambda_, options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]},
Module[ {x1,x2,x3, t, t2},{x1[t2_],x2[t2_],x3[t2_]}=InjectionME[matrix, int, fin1, t,lambda,options]/.t->t2 ; 
TableForm[Chop[{#,x1[#],x2[#],x3[#]}&/@Sort[Join[{t1},Range[10],Range[20,100,10],Range[200,1000,100],
Range[2000,10000,1000]]], 10^-45],TableHeadings\[Rule]{None,{"Days after\nlast intake","Urine\n24 hours",
"Faecal\n24 hours","Wholebody"}}]]/;m==={"Injection","MultiInputs","BioassayTable"}]; 

BioassayReportE[matrix_, int_List,  fin1_, t1_, lambda_, options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]},
Module[ {x1,x2,x3, t, t2, t3},{x1[t2_],x2[t2_],x3[t2_]}=InjectionME[matrix, int, fin1, t,lambda,options]/.t->t2 ; 
ShowLegend[LogLogPlot[{x1[t3],x2[t3]},{t3,1,t1},PlotRange\[Rule]All,
PlotStyle\[Rule]styles,AxesLabel\[Rule]{"days","Bq"},
PlotLabel\[Rule]"Multi Inputs",ImageSize\[Rule]600,DisplayFunction\[Rule]Identity],{Reverse[
MapThread[{Graphics[{Sequence@@#1,Line[{{0,0},{1,0}}]}],#2}&,{styles,{"Urine\nExcretion",
"Faecal\nExcretion"}}]],LegendPosition\[Rule]{1,-0.3},LegendSize\[Rule]{0.5,0.5},LegendShadow\[Rule]{0.02,-0.02},
LegendSpacing\[Rule]0.2}]]/;m==={"Injection","MultiInputs","GraphicReport"}]; 

(*Ingestion*)

(* v3.0: Enhanced, added Wholebody, GraphicReport and ConstantInput *)

BioassayReport[matrix_, int_, fin1_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport}/. Flatten[{options}] /. Options[BioassayReport]},
Module[{y,z, y0, z0, y1, z1, t, t2, a, b, q1, q2, q3, w, w1}, 
a = qIngestion1[matrix, int, fin1, t,0, options];
{y, z} = {qURI[t], qFEC[t]} /. a; b = {y, z};
{q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}];
w = q3 - q1 - q2;
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[-N[lambda]*t] /. t -> t2;
ExpandAll[{qDailyUrine[t1]\[Rule]y1[t1],qDailyFaecal[t1]\[Rule]z1[t1], 
qWholebody[t1]\[Rule]w1[t1]}]] /;m === {"Ingestion", "Acute", "Automatic"}];


BioassayReport[matrix_, int_, fin1_, (t1_)?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]},
Module[{y,z, y0, z0, y1, z1, t, t2, a, b, q1, q2, q3, w, w1}, 
a = qIngestion1[matrix, int, fin1, t,0, options];
{y, z} = {qURI[t], qFEC[t]} /. a; b = {y, z};
{q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}];
w = q3 - q1 - q2;
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[-N[lambda]*t] /. t -> t2;
TableForm[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[{t1}, Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 
TableHeadings -> {None, {"Days after\nstart intake", "Urine\n24 hours", "Faecal\n24 hours", 
"Wholebody"}}]] /;m === {"Ingestion","Acute","BioassayTable"}];

BioassayReport[matrix_, int_, fin1_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, y0, z0, y1, z1, t, t2, t3, a, b, q1, q2, q3, w, w1}, 
a = qIngestion1[matrix, int, fin1, t, 0, options]; {y, z} = {qURI[t], qFEC[t]} /. a; 
b = {y, z}; {q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}]; w = q3 - q1 - q2; 
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. 
t -> t2; z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2; 
w1[t2_] = w*Exp[(-lambda)*t] /. t -> t2; ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, t1}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Acute intake in t = 0", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, LegendSpacing -> 0.2}]] /; 
m === {"Ingestion","Acute","GraphicReport"}]; 

BioassayReport[matrix_, int_, fin1_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, y0, z0, y1, z1, t, t2, t3, a, b, q1, q2, q3, w, w1}, 
a = qIngestion1[matrix, int, fin1, t, 0, options]; {y, z} = {qURI[t], qFEC[t]} /. a; 
b = {y, z}; {q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}]; w = q3 - q1 - q2; 
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. 
t -> t2; z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2; 
 w1[t2_] = w*Exp[(-lambda)*t] /. t -> t2; {TableForm[({#1, y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
 Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\nintake", 
 "Urine\n24 hours", "Faecal\n24 hours", "Wholebody"}}], 
 ExpandAll[{qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1], 
qWholebody[t1] -> w1[t1]}], ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, {t3, 1, 1000}, 
PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
 PlotLabel -> "Acute intake in t = 0", ImageSize -> 600, DisplayFunction -> Identity], 
 {Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
 {styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
 LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, LegendSpacing -> 0.2}]}] /; 
     m === {"Ingestion","Acute","FullReport"}]; 

BioassayReport[matrix_, int_, fin1_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionChronic[matrix, int, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; ExpandAll[{qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1], 
qWholebody[t1] -> w1[t1]}]] /; m === {"Ingestion","Constant", "Automatic"}]; 

BioassayReport[matrix_, int_, fin1_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionChronic[matrix, int, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; TableForm[({#1, y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[{t1}, Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\n start intake", 
 "Urine\n24 hours", "Faecal\n24 hours", 
"Wholebody"}}]] /; m === {"Ingestion","Constant", "BioassayTable"}]; 

BioassayReport[matrix_, int_, fin1_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionChronic[matrix, int, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, t1}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Constant intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, LegendSpacing -> 0.2}]
]/; m === {"Ingestion","Constant", "GraphicReport"}]; 

BioassayReport[matrix_, int_, fin1_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionChronic[matrix, int, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; 
{TableForm[({#1, y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\n start intake", 
 "Urine\n24 hours", "Faecal\n24 hours", "Wholebody"}}], ExpandAll[{qDailyUrine[t1] -> y1[t1], 
 qDailyFaecal[t1] -> z1[t1], qWholebody[t1] -> w1[t1]}],
ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, 1000}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Constant intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, LegendSpacing -> 0.2}]}
]/; m === {"Ingestion", "Constant", "FullReport"}]; 

BioassayReport[matrix_, {int_, t4_}, fin1_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionContinuous[matrix, {int, t4}, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; ExpandAll[{qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1], 
qWholebody[t1] -> w1[t1]}]] /; m === {"Ingestion","Continuous", "Automatic"}]; 

BioassayReport[matrix_, {int_, t4_}, fin1_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionContinuous[matrix, {int, t4}, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; TableForm[({#1, y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[{t1}, Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\n start intake", 
 "Urine\n24 hours", "Faecal\n24 hours", 
"Wholebody"}}]] /; m === {"Ingestion","Continuous", "BioassayTable"}]; 

BioassayReport[matrix_, {int_, t4_}, fin1_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionContinuous[matrix, {int, t4}, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, t1}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Continuous intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, LegendSpacing -> 0.2}]
]/; m === {"Ingestion","Continuous", "GraphicReport"}]; 

BioassayReport[matrix_, {int_, t4_}, fin1_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionContinuous[matrix, {int, t4}, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; 
z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; 
{TableForm[({#1, y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\n start intake", 
 "Urine\n24 hours", "Faecal\n24 hours", "Wholebody"}}], ExpandAll[{qDailyUrine[t1] -> y1[t1], 
 qDailyFaecal[t1] -> z1[t1], qWholebody[t1] -> w1[t1]}],
ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, 1000}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Continuous intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, LegendSpacing -> 0.2}]}
]/; m === {"Ingestion","Continuous", "FullReport"}]; 

BioassayReport[matrix_, int_, fin1_, t1_, lambda_, options___Rule] := 
   With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. 
       Options[BioassayReport]}, qIngestion1[matrix, int, fin1, t1, lambda, options] /; 
     m === {"Ingestion", "Acute", "CompartmentContent"}]; 

BioassayReport[matrix_, int_, fin1_, t1_, lambda_, options___Rule] := 
   With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. 
       Options[BioassayReport]}, qIngestionChronic[matrix, int, fin1, t1, lambda, 
      options] /; m === {"Ingestion", "Constant", "CompartmentContent"}]; 

BioassayReport[matrix_, {int_, t_}, fin1_, t1_, lambda_, options___Rule] := 
   With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. 
       Options[BioassayReport]}, qIngestionContinuous[matrix, {int, t}, fin1, t1, 
      lambda, options] /; m === {"Ingestion", "Continuous", "CompartmentContent"}]; 

BioassayReport[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReport]}, qIngestionChronic[matrix,86400*int,fin1,t1,lambda,options]/;
        m==={"Ingestion","Acute","Disintegrations"}];

BioassayReport[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReport]}, Module[{a, x1, x2},a =qIngestionChronic[matrix,86400*int,
    fin1,t2,lambda,options];{x1[t_],x2[t_]}={First/@a,Last/@a}/.t2\[Rule]t;
    Thread[x1[t1]\[Rule]Flatten [(qContinuous[1,{#1},t,t1]&)/@x2[t]]]]/;
        m==={"Ingestion","Constant","Disintegrations"}];


IngestionM[matrix_, int_List, fin1_, t1_, lambda_, options___Rule] := 
   Module[{a, b, y, z, w, y0, z0, y1, z1, w1, t, t2, 
     list1, list2, list3}, 
    a = qIngestion1[matrix, 1, fin1, t, lambda, options]; 
     {y, z} = {qURI[t], qFEC[t]} /. a; 
     b = Plus @@ Last /@ a; w = b - y - z; 
     y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
     y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; 
     z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
     w1[t2_] = w /. t -> t2; {list1, list2} = Transpose[Append[int, {0, t1}]]; 
     list3 = Reverse[t1 + list2[[-2]] - list2]; 
     ListConvolve[list1, (y1[#1] & ) /@ list3]; 
     ExpandAll[{ListConvolve[list1, (y1[#1] & ) /@ list3][[1]], 
     ListConvolve[list1, (z1[#1] & ) /@ list3][[1]], 
     ListConvolve[list1, (w1[#1] & ) /@ list3][[1]]}]]; 

BioassayReport[matrix_, int_List,fin1_, t1_, lambda_, options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Thread[{qDailyUrine[t1], qDailyFaecal[t1], qWholebody[t1]} -> IngestionM[matrix, int, fin1, t1, lambda, options]]
 /; m === {"Ingestion", "MultiInputs", "Automatic"}]; 
 
BioassayReport[matrix_, int_List, fin1_, t1_?IntegerQ, lambda_, options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]},
Module[ {x1,x2,x3, t, t2},{x1[t2_],x2[t2_],x3[t2_]}=IngestionM[matrix, int, fin1, t,lambda,options]/.t->t2 ; 
TableForm[Chop[{#,x1[#],x2[#],x3[#]}&/@Sort[Join[{t1},Range[10],Range[20,100,10],Range[200,1000,100],
Range[2000,10000,1000]]], 10^-45],TableHeadings\[Rule]{None,{"Days after\nlast intake","Urine\n24 hours",
"Faecal\n24 hours","Wholebody"}}]]/;m==={"Ingestion","MultiInputs","BioassayTable"}]; 

BioassayReport[matrix_, int_List, fin1_, t1_, lambda_, options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]},
Module[ {x1,x2, t, t2, t3},{x1[t2_],x2[t2_],x3[t2_]}=IngestionM[matrix, int,fin1, t,lambda,options]/.t->t2 ; 
ShowLegend[LogLogPlot[{x1[t3],x2[t3]},{t3,1,t1},PlotRange\[Rule]All,
PlotStyle\[Rule]styles,AxesLabel\[Rule]{"days","Bq"},
PlotLabel\[Rule]"Multi Inputs",ImageSize\[Rule]600,DisplayFunction\[Rule]Identity],{Reverse[
MapThread[{Graphics[{Sequence@@#1,Line[{{0,0},{1,0}}]}],#2}&,{styles,{"Urine\nExcretion",
"Faecal\nExcretion"}}]],LegendPosition\[Rule]{1,-0.3},LegendSize\[Rule]{0.5,0.5},LegendShadow\[Rule]{0.02,-0.02},
LegendSpacing\[Rule]0.2}]]/;m==={"Ingestion","MultiInputs","GraphicReport"}]; 

BioassayReport[(matrixA_)?MatrixQ,int_,fin1_,t1_,lambda_,{opts___?OptionQ}]:=
    "Something is wrong";

BioassayReportE[matrix_, int_, fin1_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]},
Module[{y,z, y0, z0, y1, z1, t, t2, a, b, q1, q2, q3, w, w1}, 
a = qIngestionE1[matrix, int, fin1, t,0, options];
{y, z} = {qURI[t], qFEC[t]} /. a; b = {y, z};
{q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}];
w = q3 - q1 - q2;
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[-N[lambda]*t] /. t -> t2;
ExpandAll[{qDailyUrine[t1]\[Rule]y1[t1],qDailyFaecal[t1]\[Rule]z1[t1], 
qWholebody[t1]\[Rule]w1[t1]}]] /;m === {"Ingestion","Acute","Automatic"}];


BioassayReportE[matrix_, int_, fin1_, (t1_)?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]},
Module[{y,z, y0, z0, y1, z1, t, t2, a, b, q1, q2, q3, w, w1}, 
a = qIngestionE1[matrix, int, fin1, t,0, options];
{y, z} = {qURI[t], qFEC[t]} /. a; b = {y, z};
{q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}];
w = q3 - q1 - q2;
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[(-lambda)*t] /. t -> t2;
TableForm[Chop[({#1, y1[#1], z1[#1], w1[#1]} &) /@ Sort[Join[{t1}, Range[10],
Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000,1000]]], 10^-45], 
TableHeadings -> {None, {"Days after\nintake", "Urine\n24 hours", "Faecal\n24 hours", 
"Wholebody"}}]] /;m === {"Ingestion","Acute","BioassayTable"}];

BioassayReportE[matrix_, int_, fin1_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, y0, z0, y1, z1, t, t2, t3, a, b, q1, q2, q3, w, w1}, 
a = qIngestionE1[matrix, int, fin1, t, 0, options]; {y, z} = {qURI[t], qFEC[t]} /. a; 
b = {y, z}; {q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}]; w = q3 - q1 - q2; 
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. 
t -> t2; z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2; 
w1[t2_] = w*Exp[(-lambda)*t] /. t -> t2; ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, t1}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Acute intake in t = 0", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, LegendSpacing -> 0.2}]] /; 
m === {"Ingestion","Acute","GraphicReport"}]; 

BioassayReportE[matrix_, int_, fin1_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, y0, z0, y1, z1, t, t2, t3, a, b, q1, q2, q3, w, w1}, 
a = qIngestionE1[matrix, int, fin1, t, 0, options]; {y, z} = {qURI[t], qFEC[t]} /. a; 
b = {y, z}; {q1, q2, q3} = Flatten[{b, Plus @@ Last /@ a}]; w = q3 - q1 - q2; 
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. 
t -> t2; z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2; 
 w1[t2_] = w*Exp[(-lambda)*t] /. t -> t2; {TableForm[({#1, y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
 Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\nintake", 
 "Urine\n24 hours", "Faecal\n24 hours", "Wholebody"}}], 
 ExpandAll[{qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1], 
qWholebody[t1] -> w1[t1]}], ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, {t3, 1, 1000}, 
PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
 PlotLabel -> "Acute intake in t = 0", ImageSize -> 600, DisplayFunction -> Identity], 
 {Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
 {styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
 LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, LegendSpacing -> 0.2}]}] /; 
 m === {"Ingestion","Acute","FullReport"}]; 

BioassayReportE[matrix_, int_, fin1_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionChronicE[matrix, int, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; x1[t2_] = x /. t -> t2; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; ExpandAll[{qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1], 
qWholebody[t1] -> w1[t1]}]] /; m === {"Ingestion","Constant", "Automatic"}]; 

BioassayReportE[matrix_, int_, fin1_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionChronicE[matrix, int, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; TableForm[({#1, y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[{t1}, Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\n start intake", 
 "Urine\n24 hours", "Faecal\n24 hours", 
"Wholebody"}}]] /; m === {"Ingestion","Constant", "BioassayTable"}]; 


BioassayReportE[matrix_, int_, fin1_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionChronicE[matrix, int, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, t1}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Constant intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, LegendSpacing -> 0.2}]
]/; m === {"Ingestion","Constant", "GraphicReport"}]; 

BioassayReportE[matrix_, int_, fin1_, t1_, lambda_, options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionChronicE[matrix, int, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; 
{TableForm[({#1, y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\n start intake", 
 "Urine\n24 hours", "Faecal\n24 hours", "Wholebody"}}], ExpandAll[{qDailyUrine[t1] -> y1[t1], 
 qDailyFaecal[t1] -> z1[t1], qWholebody[t1] -> w1[t1]}],
ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, 1000}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Constant intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, LegendSpacing -> 0.2}]}
]/; m === {"Ingestion","Constant", "FullReport"}]; 

BioassayReportE[matrix_, {int_, t4_}, fin1_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionContinuousE[matrix, {int, t4}, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; ExpandAll[{qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1], 
qWholebody[t1] -> w1[t1]}]] /; m === {"Ingestion","Continuous", "Automatic"}]; 

BioassayReportE[matrix_, {int_, t4_}, fin1_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionContinuousE[matrix, {int, t4}, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; TableForm[({#1, y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[{t1}, Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\n start intake", 
 "Urine\n24 hours", "Faecal\n24 hours", 
"Wholebody"}}]] /; m === {"Ingestion","Continuous", "BioassayTable"}]; 

BioassayReportE[matrix_, {int_, t4_}, fin1_, t1_?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionContinuousE[matrix, {int, t4}, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, t1}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Continuous intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, LegendSpacing -> 0.2}]
]/; m === {"Ingestion","Continuous", "GraphicReport"}]; 

BioassayReportE[matrix_, {int_, t4_}, fin1_, t1_, lambda_,  options___Rule] := 
With[{m = {IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{y, z, w, y0, z0, y1, z1, w1, t, t2}, 
{y, z, w} = {qURI[t], qFEC[t], qWB[t]} /. qIngestionContinuousE[matrix, {int, t4}, fin1, t, lambda, 
options]; y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; 
z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_] = w /. t -> t2; 
{TableForm[({#1, y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\n start intake", 
 "Urine\n24 hours", "Faecal\n24 hours", "Wholebody"}}], ExpandAll[{qDailyUrine[t1] -> y1[t1], 
 qDailyFaecal[t1] -> z1[t1], qWholebody[t1] -> w1[t1]}],
ShowLegend[LogLogPlot[{y1[t3], z1[t3]}, 
{t3, 1, 1000}, PlotRange -> All, PlotStyle -> styles, AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Continuous intake", ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles, {"Urine\nExcretion", "Faecal\nExcretion"}}]], LegendPosition -> {1, -0.3}, 
LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, LegendSpacing -> 0.2}]}
]/; m === {"Ingestion","Continuous", "FullReport"}];

BioassayReportE[matrix_, int_, fin1_, t1_, lambda_, options___Rule] := 
   With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. 
       Options[BioassayReportE]}, qIngestionE1[matrix, int, fin1, t1, lambda, options] /; 
     m === {"Ingestion", "Acute", "CompartmentContent"}]; 

BioassayReportE[matrix_, int_, fin1_, t1_, lambda_, options___Rule] := 
   With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. 
       Options[BioassayReportE]}, qIngestionChronicE[matrix, int, fin1, t1, lambda, 
      options] /; m === {"Ingestion", "Constant", "CompartmentContent"}]; 

BioassayReportE[matrix_, {int_, t_}, fin1_, t1_, lambda_, options___Rule] := 
   With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. 
       Options[BioassayReportE]}, qIngestionContinuousE[matrix, {int, t}, fin1, t1, 
      lambda, options] /; m === {"Ingestion", "Continuous", "CompartmentContent"}]; 

BioassayReportE[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReportE]}, qIngestionChronicE[matrix,86400*int,fin1,t1,lambda,options]/;
        m==={"Ingestion","Acute","Disintegrations"}];

BioassayReportE[matrix_,int_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReportE]}, Module[{a, x1, x2},a =qIngestionChronicE[matrix,86400*int,
    fin1,t2,lambda,options];{x1[t_],x2[t_]}={First/@a,Last/@a}/.t2\[Rule]t;
    Thread[x1[t1]\[Rule]Flatten [(qContinuous[1,{#1},t,t1]&)/@x2[t]]]]/;
        m==={"Ingestion","Constant","Disintegrations"}];

IngestionME[matrix_, int_List, fin1_, t1_, lambda_, options___Rule] := 
   Module[{a, b, y, z, w, y0, z0, y1, z1, w1, t, t2, 
     list1, list2, list3}, 
    a = qIngestionE1[matrix, 1, fin1, t, lambda, options]; 
     {y, z} = {qURI[t], qFEC[t]} /. a; 
     b = Plus @@ Last /@ a; w = b - y - z; 
     y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
     y1[t2_] = y - y0*Exp[-lambda] /. t -> t2; 
     z1[t2_] = z - z0*Exp[-lambda] /. t -> t2; 
     w1[t2_] = w /. t -> t2; {list1, list2} = Transpose[Append[int, {0, t1}]]; 
     list3 = Reverse[t1 + list2[[-2]] - list2]; 
     ListConvolve[list1, (y1[#1] & ) /@ list3]; 
     ExpandAll[{ListConvolve[list1, (y1[#1] & ) /@ list3][[1]], 
     ListConvolve[list1, (z1[#1] & ) /@ list3][[1]], 
     ListConvolve[list1, (w1[#1] & ) /@ list3][[1]]}]]; 

BioassayReportE[matrix_, int_List,fin1_, t1_, lambda_, options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Thread[{qDailyUrine[t1], qDailyFaecal[t1], qWholebody[t1]} -> IngestionME[matrix, int, fin1, t1, lambda, options]]
 /; m === {"Ingestion", "MultiInputs", "Automatic"}]; 
 
BioassayReportE[matrix_, int_List, fin1_, t1_?IntegerQ, lambda_, options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]},
Module[ {x1,x2,x3, t, t2},{x1[t2_],x2[t2_],x3[t2_]}=IngestionME[matrix, int, fin1, t,lambda,options]/.t->t2 ; 
TableForm[Chop[{#,x1[#],x2[#],x3[#]}&/@Sort[Join[{t1},Range[10],Range[20,100,10],Range[200,1000,100],
Range[2000,10000,1000]]], 10^-45],TableHeadings\[Rule]{None,{"Days after\nlast intake","Urine\n24 hours",
"Faecal\n24 hours","Wholebody"}}]]/;m==={"Ingestion","MultiInputs","BioassayTable"}]; 

BioassayReportE[matrix_, int_List, fin1_, t1_, lambda_, options___Rule] := 
With[{m = {IntakeWay, IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]},
Module[ {x1,x2, t, t2, t3},{x1[t2_],x2[t2_],x3[t2_]}=IngestionME[matrix, int,fin1, t,lambda,options]/.t->t2 ; 
ShowLegend[LogLogPlot[{x1[t3],x2[t3]},{t3,1,t1},PlotRange\[Rule]All,
PlotStyle\[Rule]styles,AxesLabel\[Rule]{"days","Bq"},
PlotLabel\[Rule]"Multi Inputs",ImageSize\[Rule]600,DisplayFunction\[Rule]Identity],{Reverse[
MapThread[{Graphics[{Sequence@@#1,Line[{{0,0},{1,0}}]}],#2}&,{styles,{"Urine\nExcretion",
"Faecal\nExcretion"}}]],LegendPosition\[Rule]{1,-0.3},LegendSize\[Rule]{0.5,0.5},LegendShadow\[Rule]{0.02,-0.02},
LegendSpacing\[Rule]0.2}]]/;m==={"Ingestion","MultiInputs","GraphicReport"}]; 

BioassayReportE[(matrixA_)?MatrixQ,int_,fin1_,t1_,lambda_,{opts___?OptionQ}]:=
    "Something is wrong";

(*Inhalation*)

BioassayReport[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_,  options___Rule] := 
With[{ m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{x,y, z, y0, z0, x1, y1, z1, t, t2, a, b, q1, q2, q3, q4, w, w1}, 
a = qInhalation1[matrix, int,ifd, fra, fin1, t, 0, options];
{x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /. a; b = {x, y, z};
{q1, q2, q3, q4} = Flatten[{b, Plus @@ Last /@ a}];
w = q4 - q1 - q2 - q3 +Plus @@ Last /@ qRespTract[int, ifd, fra, t, 0, options];
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; x1[t2_] = x*Exp[-N[lambda]*t] /. t -> t2;
y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[(-lambda)*t] /. t -> t2;
ExpandAll[{qLungs[t1] -> x1[t1], qDailyUrine[t1] -> y1[t1], 
qDailyFaecal[t1] -> z1[t1], qWholebody[t1] -> w1[t1]}]] /; m === {"Inhalation","Acute", "Automatic"}];

BioassayReport[matrix_, int_, ifd_, fra_, fin1_, (t1_)?IntegerQ, lambda_,  options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{x, y, z, y0, z0, x1, y1, z1, t, t2, a, b, q1, q2, q3, q4, w, w1}, 
a = qInhalation1[matrix, int, ifd, fra, fin1, t, 0, options]; 
{x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /. a; b = {x, y, z}; 
{q1, q2, q3, q4} = Flatten[{b, Plus @@ Last /@ a}]; 
w = q4 - q1 - q2 - q3 + Plus @@ Last /@ qRespTract[int, ifd, fra, t, 0, options]; 
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; x1[t2_] = x*Exp[-N[lambda]*t] /. t -> t2; 
y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. t -> t2; 
z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2; 
w1[t2_] = w*Exp[-N[lambda]*t] /. t -> t2; 
TableForm[Chop[({#1, x1[#1], y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[{t1}, Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
Range[2000, 10000, 1000]]], 10^-45], TableHeadings -> {None, {"Days after\nintake", 
"Lung\nretention", "Urine\n24 hours", "Faecal\n24 hours", "Wholebody"}}]] /; 
m === {"Inhalation","Acute","BioassayTable"}];

BioassayReport[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{x, y, z, y0, z0, x1, y1, z1, t, t2, t3, a, b, q1, q2, q3, q4, w, w1}, 
a = qInhalation1[matrix, int, ifd, fra, fin1, t, 0, options];
{x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /. a; b = {x, y, z};
{q1, q2, q3, q4} = Flatten[{b, Plus @@ Last /@ a}];
w = q4 - q1 - q2 - q3 + Plus @@ Last /@ qRespTract[int, ifd, fra, t, 0, options];
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
x1[t2_] = x*Exp[-N[lambda]*t] /. t -> t2;
y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[-N[lambda]*t] /. t -> t2;
{TableForm[({#1, x1[#1], y1[#1], z1[#1], w1[#1]} &) /@ Sort[
Join[Range[10], Range[20, 100, 10], Range[200, 1000, 100], Range[2000, 10000, 1000]]], 
TableHeadings -> {None, {"Days after\nintake", "Lung\nretention", "Urine\n24 hours", 
"Faecal\n24 hours","Wholebody"}}], ExpandAll[{qLungs[t1] -> x1[t1], 
qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1], qWholebody[t1] -> w1[t1]}], 
ShowLegend[LogLogPlot[{x1[t3], y1[t3], z1[t3]}, {t3, 1, 1000}, PlotStyle -> styles1, 
AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Acute intake in t=0", ImageSize ->600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} &, 
{styles1, {"Lungs", "Urine\nExcretion", "Faecal\nExcretion"}}]], 
LegendPosition -> {1, -0.3}, LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]}] /; m === {"Inhalation","Acute","FullReport"}];

BioassayReport[matrix_, int_, ifd_, fra_, fin1_, t1_?IntegerQ, lambda_, options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{x, y, z, y0, z0, x1, y1, z1, t, t2, t3, a, b, q1, q2, q3, q4, w, w1}, 
a = qInhalation1[matrix, int, ifd, fra, fin1, t, 0, options];
{x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /. a; b = {x, y, z};
{q1, q2, q3, q4} = Flatten[{b, Plus @@ Last /@ a}];
w = q4 - q1 - q2 - q3 + Plus @@ Last /@ qRespTract[int, ifd, fra, t, 0, options];
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
x1[t2_] = x*Exp[-N[lambda]*t] /. t -> t2;
y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[-N[lambda]*t] /. t -> t2;
ShowLegend[LogLogPlot[{x1[t3], y1[t3], z1[t3]}, {t3, 1, t1}, PlotStyle -> styles1, 
AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Acute intake in t = 0", ImageSize ->600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} &, 
{styles1, {"Lungs", "Urine\nExcretion", "Faecal\nExcretion"}}]], 
LegendPosition -> {1, -0.3}, LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]] /; m === {"Inhalation","Acute","GraphicReport"}];

BioassayReport[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. 
Options[BioassayReport]}, 
Module[{a, x, y, z, y0, z0, x1, y1, z1, w1, t, t2}, 
a =  qInhalationChronic[matrix, int, ifd, fra, fin1, t, lambda, 
options]; {x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /.a; 
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
x1[t2_] = x /. t -> t2; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; 
z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_]= Plus@@(Last /@ a)- x/.t->t2;
ExpandAll[{qLungs[t1] -> x1[t1], 
qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1], 
qWholebody[t1] -> w1[t1]}]] /; m === {"Inhalation","Constant", "Automatic"}]; 

BioassayReport[matrix_, int_, ifd_, fra_, fin1_, t1_?IntegerQ, lambda_, options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{a, x, y, z, y0, z0, x1, y1, z1, w1, t, t2}, 
a =  qInhalationChronic[matrix, int, ifd, fra, fin1, t, lambda, 
options]; {x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /.a; 
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
x1[t2_] = x /. t -> t2; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; 
z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_]= Plus@@(Last /@ a)- x/.t->t2;
TableForm[({#1, x1[#1], y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[{t1}, Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\nintake", 
"Thoracic\nLung", "Urine\n24 hours", "Faecal\n24 hours", "Wholebody"}}]] /; 
 m === {"Inhalation","Constant", "BioassayTable"}]; 

BioassayReport[matrix_, int_, ifd_, fra_, fin1_, t1_?IntegerQ, lambda_, options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{a, x, y, z, y0, z0, x1, y1, z1, w1, t, t2}, 
a =  qInhalationChronic[matrix, int, ifd, fra, fin1, t, lambda, 
options]; {x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /.a; 
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
x1[t2_] = x /. t -> t2; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; 
z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_]= Plus@@(Last /@ a)- x/.t->t2;
ShowLegend[LogLogPlot[{x1[t2], y1[t2], z1[t2]}, {t2, 1, t1}, PlotStyle -> styles1, 
AxesLabel -> {"days", "Bq"}, PlotLabel -> "Constant intake", 
ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles1, {"Lungs", "Urine\nExcretion", "Faecal\nExcretion"}}]], 
LegendPosition -> {1, -0.3}, LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]] /; m === {"Inhalation","Constant", "GraphicReport"}]; 

BioassayReport[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, options___Rule] := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReport]}, 
Module[{a, x, y, z, y0, z0, x1, y1, z1, w1, t, t2}, 
a =  qInhalationChronic[matrix, int, ifd, fra, fin1, t, lambda, 
options]; {x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /.a; 
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
x1[t2_] = x /. t -> t2; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; 
z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_]= Plus@@(Last /@ a)- x/.t->t2;
{ TableForm[({#1, x1[#1], y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[ Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\nintake", 
"Thoracic\nLung", "Urine\n24 hours", "Faecal\n24 hours", "Wholebody"}}],
ExpandAll[{qLungs[t1] -> x1[t1], qDailyUrine[t1] -> 
y1[t1], qDailyFaecal[t1] -> z1[t1], qWholebody[t1] -> w1[t1]}],
ShowLegend[LogLogPlot[{x1[t2], y1[t2], z1[t2]}, {t2, 1, 1000}, PlotStyle -> styles1, 
AxesLabel -> {"days", "Bq"}, PlotLabel -> "Constant intake", 
ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles1, {"Lungs", "Urine\nExcretion", "Faecal\nExcretion"}}]], 
LegendPosition -> {1, -0.3}, LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]}] /; m === {"Inhalation","Constant", "FullReport"}]; 

BioassayReport[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
   BioassayReport]},qInhalation["Acute", matrix,int,ifd,fra,fin1,t1,lambda,options]/;
        m==={"Inhalation","Acute","CompartmentContent"}];

BioassayReport[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReport]}, qInhalationChronic[matrix,int,ifd,fra,fin1,t1,lambda,options]/;
        m==={"Inhalation","Constant","CompartmentContent"}];

BioassayReport[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReport]}, qInhalationChronic[matrix,86400*int,ifd,fra,fin1,t1,lambda,options]/;
        m==={"Inhalation","Acute","Disintegrations"}];

BioassayReport[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReport]}, Module[{a, x1, x2},a =qInhalationChronic[matrix,86400*int,
    ifd,fra,fin1,t2,lambda,options];{x1[t_],x2[t_]}={First/@a,Last/@a}/.t2\[Rule]t;
    Thread[x1[t1]\[Rule]Flatten [(qContinuous[1,{#1},t,t1]&)/@x2[t]]]]/;
        m==={"Inhalation","Constant","Disintegrations"}];


InhalationM[matrix_,int_List,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
    Module[{x,y,z,y0,z0,x1,y1,z1,t,t2,a,b,q1,q2,q3,q4,w,w1,list1,list2,list3},
      a=qInhalation1[matrix,1,ifd,fra,fin1,t,0,options];
      {x,y,z}={qLungs[t],qURI[t],qFEC[t]}/.a;
      b=Plus@@Last/@a;b={x,y,z};
      {q1,q2,q3,q4}=Flatten[{b,Plus@@Last/@a}];
      w=q4-q1-q2-q3+Plus@@Last/@Drop[qRT[1,ifd,fra,t,0,options],-3];
      y0=y/.t\[Rule]t-1;z0=z/.t\[Rule]t-1;
      x1[t2_]=x*Exp[-N[lambda]*t]/.t\[Rule]t2;
      y1[t2_]=(y-y0)*Exp[-N[lambda]*t]/.t\[Rule]t2;
      z1[t2_]=(z-z0)*Exp[-N[lambda]*t]/.t\[Rule]t2;
      w1[t2_]=w*Exp[-N[lambda]*t]/.t\[Rule]t2;
      {list1,list2}=Transpose[Append[int,{0,t1}]];
      list3=Reverse[t1+list2[[-2]]-list2];
      ListConvolve[list1,(y1[#1]&)/@list3];
      ExpandAll[{ListConvolve[list1,(x1[#1]&)/@list3][[1]],
      ListConvolve[list1,(y1[#1]&)/@list3][[1]],ListConvolve[list1,(z1[#1]&)/@list3][[1]],
      ListConvolve[list1,(w1[#1]&)/@list3][[1]]}]];

BioassayReport[matrix_,int_List,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]},
Thread[{qLungs[t1],qDailyUrine[t1],qDailyFaecal[t1],qWholebody[t1]}\[Rule]
InhalationM[matrix,int,ifd,fra,fin1,t1,lambda,options]]/;m==={"Inhalation","MultiInputs","Automatic"}];

BioassayReport[matrix_,int_List,ifd_,fra_,fin1_,t1_?IntegerQ,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]},
Module[{x1,x2,x3,x4,t,t2},{x1[t2_],x2[t2_],x3[t2_],x4[t2_]}=
InhalationM[matrix,int,ifd,fra, fin1, t,lambda,options]/.t\[Rule]t2;
TableForm[Chop[{#,x1[#],x2[#],x3[#], x4[#]}&/@Sort[Join[{t1},Range[10],Range[20,100,10],Range[200,1000,100],
Range[2000,10000,1000]]], 10^-45],TableHeadings\[Rule]{None,{"Days after\nlast intake","Lung",
"Urine\n24 hours","Faecal\n24 hours","Wholebody"}}]]/;m==={"Inhalation","MultiInputs","BioassayTable"}];

BioassayReport[matrix_,int_List,ifd_,fra_,fin1_,t1_?IntegerQ,lambda_,
options___Rule]:=With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReport]},
Module[{x1,x2,x3,x4,t,t2, t3},{x1[t2_],x2[t2_],x3[t2_],x4[t2_]}=
InhalationM[matrix,int,ifd,fra,fin1,t,lambda,options]/.t\[Rule]t2;
ShowLegend[LogLogPlot[{x1[t3],x2[t3],x3[t3]},{t3,1,t1},
PlotStyle\[Rule]styles1,AxesLabel\[Rule]{"days","Bq"},
PlotLabel\[Rule]"Multiple intake",ImageSize\[Rule]600,
DisplayFunction\[Rule]Identity],{Reverse[MapThread[{Graphics[{Sequence@@#1,
Line[{{0,0},{1,0}}]}],#2}&,{styles1,{"Lungs","Urine\nExcretion","Faecal\nExcretion"}}]],
LegendPosition\[Rule]{1,-0.3},LegendSize\[Rule]{0.5,0.5},
LegendShadow\[Rule]{0.02,-0.02},LegendSpacing\[Rule]0.2}]]/;m==={"Inhalation","MultiInputs","GraphicReport"}];


BioassayReport[(matrixA_)?MatrixQ, int_, ifd_List, fra_List, fin1_, t1_, lambda_, 
    {(opts___)?OptionQ}]:="Something is wrong"; 

BioassayReportE[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, options___Rule]/; fin1 < 1 := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{x,y, z, y0, z0, x1, y1, z1, t, t2, a, b, q1, q2, q3, q4, w, w1}, 
a = qInhalationE1[matrix, int,ifd, fra, fin1, t, 0, options];
  {x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /. a; b = {x, y, z};
  {q1, q2, q3, q4} = Flatten[{b, Plus @@ Last /@ a}];
  w = q4 - q1 - q2 - q3 +Plus @@ Last /@ qRespTract[int, ifd, fra, t, 0, options];
  y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; x1[t2_] = x*Exp[(-lambda)*t] /. t -> t2;
  y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. t -> t2;
  z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2;
  w1[t2_] = w*Exp[(-lambda)*t] /. t -> t2;
   ExpandAll[{qLungs[t1] -> x1[t1], qDailyUrine[t1] -> y1[t1], 
   qDailyFaecal[t1] -> z1[t1], qWholebody[t1] -> w1[t1]}]] /; m === {"Inhalation","Acute","Automatic"}];

BioassayReportE[matrix_, int_, ifd_, fra_, fin1_, (t1_)?IntegerQ, lambda_, options___Rule]/; fin1 < 1 := 
  With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
   Module[{x, y, z, y0, z0, x1, y1, z1, t, t2, a, b, q1, q2, q3, q4, w, w1}, 
     a = qInhalationE1[matrix, int, ifd, fra, fin1, t, 0, options]; 
      {x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /. a; b = {x, y, z}; 
      {q1, q2, q3, q4} = Flatten[{b, Plus @@ Last /@ a}]; 
      w = q4 - q1 - q2 - q3 + Plus @@ Last /@ qRespTract[int, ifd, fra, t, 0, options]; 
      y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; x1[t2_] = x*Exp[(-lambda)*t] /. t -> t2; 
      y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. t -> t2; 
      z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2; 
      w1[t2_] = w*Exp[(-lambda)*t] /. t -> t2; 
      TableForm[Chop[({#1, x1[#1], y1[#1], z1[#1], w1[#1]} & ) /@ 
        Sort[Join[{t1}, Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
          Range[2000, 10000, 1000]]], 10^-45], TableHeadings -> {None, {"Days after\nintake", 
          "Lung\nretention", "Urine\n24 hours", "Faecal\n24 hours", "Wholebody"}}]] /; 
    m === {"Inhalation","Acute","BioassayTable"}];

BioassayReportE[matrix_, int_, ifd_, fra_, fin1_, t1_?IntegerQ, lambda_, options___Rule]/; fin1 <1 := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{x, y, z, y0, z0, x1, y1, z1, t, t2, t3, a, b, q1, q2, q3, q4, w, w1}, 
a = qInhalationE1[matrix, int, ifd, fra, fin1, t, 0, options];
{x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /. a; b = {x, y, z};
{q1, q2, q3, q4} = Flatten[{b, Plus @@ Last /@ a}];
w = q4 - q1 - q2 - q3 + Plus @@ Last /@ qRespTract[int, ifd, fra, t, 0, options];
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
x1[t2_] = x*Exp[-N[lambda]*t] /. t -> t2;
y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[-N[lambda]*t] /. t -> t2;
ShowLegend[LogLogPlot[{x1[t3], y1[t3], z1[t3]}, {t3, 1, t1}, PlotStyle -> styles1, 
AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Acute intake in t = 0", ImageSize ->600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} &, 
{styles1, {"Lungs", "Urine\nExcretion", "Faecal\nExcretion"}}]], 
LegendPosition -> {1, -0.3}, LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]] /; m ==={"Inhalation","Acute", "GraphicReport"}];

BioassayReportE[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, options___Rule] /; fin1 <1:= 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{x, y, z, y0, z0, x1, y1, z1, t, t2, t3, a, b, q1, q2, q3, q4, w, w1}, 
a = qInhalationE1[matrix, int, ifd, fra, fin1, t, 0, options];
{x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /. a; b = {x, y, z};
{q1, q2, q3, q4} = Flatten[{b, Plus @@ Last /@ a}];
w = q4 - q1 - q2 - q3 + Plus @@ Last /@ qRespTract[int, ifd, fra, t, 0, options];
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1;
x1[t2_] = x*Exp[(-lambda)*t] /. t -> t2;
y1[t2_] = (y - y0)*Exp[-N[lambda]*t] /. t -> t2;
z1[t2_] = (z - z0)*Exp[-N[lambda]*t] /. t -> t2;
w1[t2_] = w*Exp[-N[lambda]*t] /. t -> t2;
{TableForm[({#1, x1[#1], y1[#1], z1[#1], w1[#1]} &) /@ Sort[
Join[Range[10], Range[20, 100, 10], Range[200, 
1000, 100], Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\nintake", 
"Lung\nretention", "Urine\n24 hours", "Faecal\n24hours","Wholebody"}}], 
ExpandAll[{qLungs[t1] -> x1[t1], qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1],
qWholebody[t1] -> w1[t1]}], 
ShowLegend[LogLogPlot[{x1[t3], y1[t3], z1[t3]}, {t3, 1, 1000}, PlotStyle -> styles1, 
AxesLabel -> {"days", "Bq"}, 
PlotLabel -> "Acute intake in t = 0", ImageSize ->600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} &, 
{styles1, {"Lungs", "Urine\nExcretion", "Faecal\nExcretion"}}]], 
LegendPosition -> {1, -0.3}, LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]}] /; m === {"Inhalation","Acute","FullReport"}];

BioassayReportE[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, options___Rule]/; fin1 <1 := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. 
Options[BioassayReportE]}, 
Module[{a, x, y, z, y0, z0, x1, y1, z1, w1, t, t2}, 
a =  qInhalationChronicE[matrix, int, ifd, fra, fin1, t, lambda, 
options]; {x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /.a; 
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
x1[t2_] = x /. t -> t2; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; 
z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_]= Plus@@(Last /@ a)- x/.t->t2;
ExpandAll[{qLungs[t1] -> x1[t1], 
qDailyUrine[t1] -> y1[t1], qDailyFaecal[t1] -> z1[t1], 
qWholebody[t1] -> w1[t1]}]] /; m === {"Inhalation","Constant", "Automatic"}]; 

BioassayReportE[matrix_, int_, ifd_, fra_, fin1_, t1_?IntegerQ, lambda_, options___Rule]/; fin1 <1 := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{a, x, y, z, y0, z0, x1, y1, z1, w1, t, t2}, 
a =  qInhalationChronicE[matrix, int, ifd, fra, fin1, t, lambda, 
options]; {x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /.a; 
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
x1[t2_] = x /. t -> t2; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; 
z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_]= Plus@@(Last /@ a)- x/.t->t2;
TableForm[({#1, x1[#1], y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[{t1}, Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\nintake", 
"Thoracic\nLung", "Urine\n24 hours", "Faecal\n24 hours", "Wholebody"}}]] /; 
 m === {"Inhalation","Constant", "BioassayTable"}]; 

BioassayReportE[matrix_, int_, ifd_, fra_, fin1_, t1_?IntegerQ, lambda_, options___Rule]/; fin1 <1 := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{a, x, y, z, y0, z0, x1, y1, z1, w1, t, t2}, 
a =  qInhalationChronicE[matrix, int, ifd, fra, fin1, t, lambda, 
options]; {x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /.a; 
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
x1[t2_] = x /. t -> t2; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; 
z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_]= Plus@@(Last /@ a)- x/.t->t2;
ShowLegend[LogLogPlot[{x1[t2], y1[t2], z1[t2]}, {t2, 1, t1}, PlotStyle -> styles1, 
AxesLabel -> {"days", "Bq"}, PlotLabel -> "Constant intake", 
ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles1, {"Lungs", "Urine\nExcretion", "Faecal\nExcretion"}}]], 
LegendPosition -> {1, -0.3}, LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]] /; m === {"Inhalation","Constant", "GraphicReport"}]; 

BioassayReportE[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, options___Rule]/; fin1 <1 := 
With[{m = {IntakeWay,IntakeType, ShowReport} /. Flatten[{options}] /. Options[BioassayReportE]}, 
Module[{a, x, y, z, y0, z0, x1, y1, z1, w1, t, t2}, 
a =  qInhalationChronicE[matrix, int, ifd, fra, fin1, t, lambda, 
options]; {x, y, z} = {qLungs[t], qURI[t], qFEC[t]} /.a; 
y0 = y /. t -> t - 1; z0 = z /. t -> t - 1; 
x1[t2_] = x /. t -> t2; 
y1[t2_] = y - y0*Exp[-N[lambda]] /. t -> t2; 
z1[t2_] = z - z0*Exp[-N[lambda]] /. t -> t2; 
w1[t2_]= Plus@@(Last /@ a)- x/.t->t2;
{ TableForm[({#1, x1[#1], y1[#1], z1[#1], w1[#1]} & ) /@ 
Sort[Join[ Range[10], Range[20, 100, 10], Range[200, 1000, 100], 
Range[2000, 10000, 1000]]], TableHeadings -> {None, {"Days after\nintake", 
"Thoracic\nLung", "Urine\n24 hours", "Faecal\n24 hours", "Wholebody"}}],
ExpandAll[{qLungs[t1] -> x1[t1], qDailyUrine[t1] -> 
y1[t1], qDailyFaecal[t1] -> z1[t1], qWholebody[t1] -> w1[t1]}],
ShowLegend[LogLogPlot[{x1[t2], y1[t2], z1[t2]}, {t2, 1, 1000}, PlotStyle -> styles1, 
AxesLabel -> {"days", "Bq"}, PlotLabel -> "Constant intake", 
ImageSize -> 600, DisplayFunction -> Identity], 
{Reverse[MapThread[{Graphics[{Sequence @@ #1, Line[{{0, 0}, {1, 0}}]}], #2} & , 
{styles1, {"Lungs", "Urine\nExcretion", "Faecal\nExcretion"}}]], 
LegendPosition -> {1, -0.3}, LegendSize -> {0.5, 0.5}, LegendShadow -> {0.02, -0.02}, 
LegendSpacing -> 0.2}]}] /; m === {"Constant", "FullReport"}];

BioassayReportE[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReportE]},qInhalationE["Acute",matrix,int,ifd,fra,fin1,t1,lambda,options]/;
        m==={"Inhalation","Acute","CompartmentContent"}];

BioassayReportE[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReportE]}, qInhalationChronicE[matrix,int,ifd,fra,fin1,t1,lambda,options]/;
        m==={"Inhalation","Constant","CompartmentContent"}];

BioassayReportE[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReportE]}, qInhalationChronicE[matrix,86400*int,ifd,fra,fin1,t1,lambda,options]/;
        m==={"Inhalation","Acute","Disintegrations"}];

BioassayReportE[matrix_,int_,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
    With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[
    BioassayReportE]}, Module[{a, x1, x2},a =qInhalationChronicE[matrix,86400*int,ifd,fra,fin1,
    t2,lambda,options];{x1[t_],x2[t_]}={First/@a,Last/@a}/.t2\[Rule]t;
    Thread[x1[t1]\[Rule]Flatten [(qContinuous[1,{#1},t,t1]&)/@x2[t]]]]/;
        m==={"Inhalation","Constant","Disintegrations"}];

InhalationME[matrix_,int_List,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
    Module[{x,y,z,y0,z0,x1,y1,z1,t,t2,a,b,q1,q2,q3,q4,w,w1,list1,list2,list3},
      a=qInhalationE1[matrix,1,ifd,fra,fin1,t,0,options];
      {x,y,z}={qLungs[t],qURI[t],qFEC[t]}/.a;
      b=Plus@@Last/@a;b={x,y,z};
      {q1,q2,q3,q4}=Flatten[{b,Plus@@Last/@a}];
      w=q4-q1-q2-q3+Plus@@Last/@Drop[qRT[1,ifd,fra,t,0,options],-3];
      y0=y/.t\[Rule]t-1;z0=z/.t\[Rule]t-1;
      x1[t2_]=x*Exp[-N[lambda]*t]/.t\[Rule]t2;
      y1[t2_]=(y-y0)*Exp[-N[lambda]*t]/.t\[Rule]t2;
      z1[t2_]=(z-z0)*Exp[-N[lambda]*t]/.t\[Rule]t2;
      w1[t2_]=w*Exp[-N[lambda]*t]/.t\[Rule]t2;
      {list1,list2}=Transpose[Append[int,{0,t1}]];
      list3=Reverse[t1+list2[[-2]]-list2];
      ListConvolve[list1,(y1[#1]&)/@list3];
      ExpandAll[{ListConvolve[list1,(x1[#1]&)/@list3][[1]],
      ListConvolve[list1,(y1[#1]&)/@list3][[1]],ListConvolve[list1,(z1[#1]&)/@list3][[1]],
      ListConvolve[list1,(w1[#1]&)/@list3][[1]]}]];

BioassayReportE[matrix_,int_List,ifd_,fra_,fin1_,t1_,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReportE]},
Thread[{qLungs[t1],qDailyUrine[t1],qDailyFaecal[t1],qWholebody[t1]}\[Rule]
InhalationME[matrix,int,ifd,fra,fin1,t1,lambda,options]]/;m==={"Inhalation","MultiInputs","Automatic"}];

BioassayReportE[matrix_,int_List,ifd_,fra_,fin1_,t1_?IntegerQ,lambda_,options___Rule]:=
With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReportE]},
Module[{x1,x2,x3,x4,t,t2},{x1[t2_],x2[t2_],x3[t2_],x4[t2_]}=
InhalationME[matrix,int,ifd,fra, fin1, t,lambda,options]/.t\[Rule]t2;
TableForm[Chop[{#,x1[#],x2[#],x3[#], x4[#]}&/@Sort[Join[{t1},Range[10],Range[20,100,10],Range[200,1000,100],
Range[2000,10000,1000]]], 10^-45],TableHeadings\[Rule]{None,{"Days after\nlast intake","Lung",
"Urine\n24 hours","Faecal\n24 hours","Wholebody"}}]]/;m==={"Inhalation","MultiInputs","BioassayTable"}];

BioassayReportE[matrix_,int_List,ifd_,fra_,fin1_,t1_?IntegerQ,lambda_,
options___Rule]:=With[{m={IntakeWay,IntakeType,ShowReport}/.Flatten[{options}]/.Options[BioassayReportE]},
Module[{x1,x2,x3,x4,t,t2, t3},{x1[t2_],x2[t2_],x3[t2_],x4[t2_]}=
InhalationME[matrix,int,ifd,fra,fin1,t,lambda,options]/.t\[Rule]t2;
ShowLegend[LogLogPlot[{x1[t3],x2[t3],x3[t3]},{t3,1,t1},
PlotStyle\[Rule]styles1,AxesLabel\[Rule]{"days","Bq"},
PlotLabel\[Rule]"Multiple intake",ImageSize\[Rule]600,
DisplayFunction\[Rule]Identity],{Reverse[MapThread[{Graphics[{Sequence@@#1,
Line[{{0,0},{1,0}}]}],#2}&,{styles1,{"Lungs","Urine\nExcretion","Faecal\nExcretion"}}]],
LegendPosition\[Rule]{1,-0.3},LegendSize\[Rule]{0.5,0.5},
LegendShadow\[Rule]{0.02,-0.02},LegendSpacing\[Rule]0.2}]]/;m==={"Inhalation","MultiInputs","GraphicReport"}];

BioassayReportE[matrix_, int_, ifd_, fra_, fin1_, t1_, lambda_, options___Rule]/;fin1 >= 1 := 
"f1 must be less than 1";

BioassayReportE[(matrixA_)?MatrixQ, int_, ifd_List, fra_List, fin1_, t1_, lambda_, 
    {(opts___)?OptionQ}]:="Something is wrong"; 

(*Added in v 3.3*)

qRandom[b1_, T_, inp_, mu_, sig_, z_, n_Integer, m_Integer, t_:t] := 
   Module[{r, rc, T1, list1, r1, r1b, r2, t1, u1, uu1, ub, int}, r[t1_] = inp /. t -> t1; 
     rc[T1_] = (b1/T)*Integrate[r[t1], {t1, T1, T + T1}]; list1 = PadRight[{1}, n, 
     PadRight[PadRight[{1}, m, 1], 7]]; 
     r1 = ListConvolve[list1, (r[#1] & ) /@ Range[n], {-n, 1}, 0]; 
     r1b = (rc[#1] & ) /@ Range[n]; u1 = mu*r1 + r1b; 
     uu1 = Drop[u1, 49]; r2 = ListConvolve[list1, (r[#1]^2 & ) /@ Range[n], {-n, 1}, 0]; ub = sig^2*r2; 
     int = Drop[ListCorrelate[{1, 1, 1, 1, 1, 1, 1}/7, z*Sqrt[ub], {-7, 7}], 43]; 
     Transpose[{Drop[Range[n], 49], uu1, int, uu1 + int, uu1 - int}]]; 

qRandom[inp_, mu_, sig_, z_, n_Integer, m_Integer, t_:t] := 
   Module[{r, list1, r1, r2, t1, u1, uu1, ub, int}, r[t1_] = inp /. t -> t1; 
     list1 = PadRight[{1}, n, PadRight[PadRight[{1}, m, 1], 7]]; 
     r1 = ListConvolve[list1, (r[#1] & ) /@ Range[n], {-n, 1}, 0]; 
     u1 = mu*r1; 
     uu1 = Drop[u1, 49]; r2 = ListConvolve[list1, (r[#1]^2 & ) /@ Range[n], {-n, 1}, 0]; ub = sig^2*r2; 
     int = Drop[ListCorrelate[{1, 1, 1, 1, 1, 1, 1}/7, z*Sqrt[ub], {-7, 7}], 43]; 
     Transpose[{Drop[Range[n], 49], uu1, int, uu1 + int, uu1 - int}]]; 
End[]

Protect[qContinuous, icrp30Model, qRT, qRTandGI, qGI, qInhalation, 
qInhalationE, qIngestion, qIngestionE, qInjection, qInjectionE, BioassayReport, 
BioassayReportE, qWholebody, qWholebodyE, qRandom];

Print["Humorap 3.3 b1 2005-06-17"]

EndPackage[]
